from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.employer_template_type import EmployerTemplateType
from ..types import UNSET, Unset

T = TypeVar("T", bound="EmployerTemplate")


@attr.s(auto_attribs=True)
class EmployerTemplate:
    """
    Attributes:
        type (Union[Unset, EmployerTemplateType]):
        description (Union[Unset, None, str]): Explains the purpose of this template
        is_custom (Union[Unset, bool]): Indicates whether or not this template has been changed from the default
            content.
        content (Union[Unset, None, str]):
        subject (Union[Unset, None, str]):
        default_content (Union[Unset, None, str]): [readonly] The default content for this template
        default_subject (Union[Unset, None, str]): [readonly] The default subject for this template
        has_subject (Union[Unset, bool]): [readonly] If false then the Subject property is not relevant for this
            template type
        tokens (Union[Unset, None, List[str]]): [readonly] A list of tokens that can be used with this template.
            For example, if 'Employee.FirstName' is in the list then use '{Employee.FirstName}' in the Content or Subject
            and
            it'll be replaced with the relevant value when the template is parsed.
        tokens_expanded (Union[Unset, bool]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    type: Union[Unset, EmployerTemplateType] = UNSET
    description: Union[Unset, None, str] = UNSET
    is_custom: Union[Unset, bool] = UNSET
    content: Union[Unset, None, str] = UNSET
    subject: Union[Unset, None, str] = UNSET
    default_content: Union[Unset, None, str] = UNSET
    default_subject: Union[Unset, None, str] = UNSET
    has_subject: Union[Unset, bool] = UNSET
    tokens: Union[Unset, None, List[str]] = UNSET
    tokens_expanded: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        description = self.description
        is_custom = self.is_custom
        content = self.content
        subject = self.subject
        default_content = self.default_content
        default_subject = self.default_subject
        has_subject = self.has_subject
        tokens: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.tokens, Unset):
            if self.tokens is None:
                tokens = None
            else:
                tokens = self.tokens

        tokens_expanded = self.tokens_expanded
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if description is not UNSET:
            field_dict["description"] = description
        if is_custom is not UNSET:
            field_dict["isCustom"] = is_custom
        if content is not UNSET:
            field_dict["content"] = content
        if subject is not UNSET:
            field_dict["subject"] = subject
        if default_content is not UNSET:
            field_dict["defaultContent"] = default_content
        if default_subject is not UNSET:
            field_dict["defaultSubject"] = default_subject
        if has_subject is not UNSET:
            field_dict["hasSubject"] = has_subject
        if tokens is not UNSET:
            field_dict["tokens"] = tokens
        if tokens_expanded is not UNSET:
            field_dict["tokensExpanded"] = tokens_expanded
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, EmployerTemplateType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = EmployerTemplateType(_type)

        description = d.pop("description", UNSET)

        is_custom = d.pop("isCustom", UNSET)

        content = d.pop("content", UNSET)

        subject = d.pop("subject", UNSET)

        default_content = d.pop("defaultContent", UNSET)

        default_subject = d.pop("defaultSubject", UNSET)

        has_subject = d.pop("hasSubject", UNSET)

        tokens = cast(List[str], d.pop("tokens", UNSET))

        tokens_expanded = d.pop("tokensExpanded", UNSET)

        id = d.pop("id", UNSET)

        employer_template = cls(
            type=type,
            description=description,
            is_custom=is_custom,
            content=content,
            subject=subject,
            default_content=default_content,
            default_subject=default_subject,
            has_subject=has_subject,
            tokens=tokens,
            tokens_expanded=tokens_expanded,
            id=id,
        )

        return employer_template



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.nvr_employee import NvrEmployee
from ..types import UNSET, Unset

T = TypeVar("T", bound="NvrRequest")


@attr.s(auto_attribs=True)
class NvrRequest:
    """
    Attributes:
        employee (Union[Unset, None, List[NvrEmployee]]):
        related_tax_year (Union[Unset, None, str]):
    """

    employee: Union[Unset, None, List[NvrEmployee]] = UNSET
    related_tax_year: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.employee, Unset):
            if self.employee is None:
                employee = None
            else:
                employee = []
                for employee_item_data in self.employee:
                    employee_item = employee_item_data.to_dict()

                    employee.append(employee_item)

        related_tax_year = self.related_tax_year

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee is not UNSET:
            field_dict["employee"] = employee
        if related_tax_year is not UNSET:
            field_dict["relatedTaxYear"] = related_tax_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employee = []
        _employee = d.pop("employee", UNSET)
        for employee_item_data in _employee or []:
            employee_item = NvrEmployee.from_dict(employee_item_data)

            employee.append(employee_item)

        related_tax_year = d.pop("relatedTaxYear", UNSET)

        nvr_request = cls(
            employee=employee,
            related_tax_year=related_tax_year,
        )

        return nvr_request



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.pay_run_cost_summary import PayRunCostSummary
from ..models.recoverable_amounts import RecoverableAmounts
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayRunTotals")


@attr.s(auto_attribs=True)
class PayRunTotals:
    """Used to represent totals for a PayRun or PayRunEntry.
    If a value is 0 then it will not be shown in the JSON.

        Attributes:
            basic_pay (Union[Unset, float]): [readonly] The amount to be paid to this Employee as a result of the PayOptions
                that have been set
            gross (Union[Unset, float]): [readonly] Gross pay
            gross_for_ni (Union[Unset, float]): [readonly] The amount of the Gross that is subject to NI
            gross_not_subject_to_employers_ni (Union[Unset, float]): [readonly] The amount of the Gross that is not subject
                to Employers NI.
                This is the same as GrossForNI where the employees NI Table has an Er contribution
            gross_for_tax (Union[Unset, float]): [readonly] The amount of the Gross that is subject to PAYE
            employer_ni (Union[Unset, float]): [readonly]
            employee_ni (Union[Unset, float]): [readonly]
            employer_ni_off_payroll (Union[Unset, float]): [readonly] The amount included in EmployerNi that is in relation
                to Off-Payroll Workers
            real_time_class_1a_ni (Union[Unset, float]): [readonly]
            tax (Union[Unset, float]): [readonly]
            net_pay (Union[Unset, float]): [readonly] The Net Pay for this Employee
            adjustments (Union[Unset, float]): [readonly] The value of adjustments made to the Net Pay (Non taxable/NIable
                additions/deductions)
            additions (Union[Unset, float]): The value of all additions.  This minus Deductions should equal TakeHomePay
            deductions (Union[Unset, float]): The value of all deductions.  Additions minus This value should equal
                TakeHomePay
            take_home_pay (Union[Unset, float]): [readonly] The amount this Employee takes home
            non_tax_or_nic_pmt (Union[Unset, float]): [readonly] The value of any payments being made to this Employee that
                aren't being subjected to PAYE or NI
            items_subject_to_class_1nic (Union[Unset, float]): [readonly] Items subject to Class 1 NIC but not taxed under
                PAYE regulations excluding pension contributions
            dedns_from_net_pay (Union[Unset, float]): [readonly] The value of any deductions being made to the Net Pay for
                this Employee
            tcp_tcls (Union[Unset, float]): [readonly] Value of payments marked as Trivial Commutation Payment (A - TCLS)
            tcp_pp (Union[Unset, float]): [readonly] Value of payments marked as Trivial Commutation Payment (B - Personal
                Pension)
            tcp_op (Union[Unset, float]): [readonly] Value of payments marked as Trivial Commutation Payment (C -
                Occupational Pension)
            flexi_dd_death (Union[Unset, float]): [readonly] Value of payments marked as flexibly accessing death benefit
                (taxable)
            flexi_dd_death_non_tax (Union[Unset, float]): [readonly] Value of payments marked as flexibly accessing death
                benefit (non taxable)
            flexi_dd_pension (Union[Unset, float]): [readonly] Value of payments marked as flexibly accessing pension
                (taxable)
            flexi_dd_pension_non_tax (Union[Unset, float]): [readonly] Value of payments marked as flexibly accessing
                pension (non taxable)
            smp (Union[Unset, float]): [readonly] Statutory Maternity Pay
            spp (Union[Unset, float]): [readonly] Statutory Paternity Pay
            sap (Union[Unset, float]): [readonly] Statutory Adoption Pay
            shpp (Union[Unset, float]): [readonly] Statutory Shared Parental Pay
            spbp (Union[Unset, float]): [readonly] Statutory Parental Bereavement Pay
            ssp (Union[Unset, float]): [readonly] Statutory Sick  Pay
            student_loan_recovered (Union[Unset, float]): [readonly]
            postgrad_loan_recovered (Union[Unset, float]): [readonly]
            pensionable_earnings (Union[Unset, float]): [readonly] The amount of the Gross that is subject to Pension
                Deductions.
                If the Pension Scheme uses Qualifying Earnings (upper and lower limits) then this value is before those are
                applied
            pensionable_pay (Union[Unset, float]): [readonly] The amount of the Gross that pension calculations are based on
                after taking into account Upper and Lower Limits for the WorkerGroup.
            non_tierable_pay (Union[Unset, float]): [readonly] The value of any pay that shouldn't count towards determining
                a pension tier.
            employee_pension_contribution (Union[Unset, float]): [readonly] The value of the Pension Contribution being made
                by this Employee, excluding any Additional Voluntary Contributions
            employee_pension_contribution_avc (Union[Unset, float]): [readonly] The value of the Pension Contribution being
                made by this Employee as an Additional Voluntary Contribution
            employer_pension_contribution (Union[Unset, float]): [readonly] The value of the Pension Contribution being made
                by the Employer for this Employee
            empee_pen_contribns_not_paid (Union[Unset, float]): [readonly] Value of employee pension contributions that are
                not paid under 'net pay arrangements', including any AVC
            empee_pen_contribns_paid (Union[Unset, float]): [readonly] Value of employee pension contributions paid under
                'net pay arrangements', including any AVC
            attachment_order_deductions (Union[Unset, float]): [readonly] Value of deductions made due to AttachmentOrders
            cis_deduction (Union[Unset, float]): [readonly] Value of any CIS Deduction made
            cis_vat (Union[Unset, float]): [readonly] Value of any VAT paid to CIS Subcontractor
            cis_umbrella_fee (Union[Unset, float]): [readonly] Value of any pre-tax fee charged to the CIS Subcontractor for
                processing the payment
            cis_umbrella_fee_post_tax (Union[Unset, float]): [readonly] Value of any post-tax fee charged to the CIS
                Subcontractor for processing the payment
            pbik (Union[Unset, float]): [readonly] Value of any Payrolled Benefits In Kind
            maps_miles (Union[Unset, int]): [readonly] The number of miles paid for Mileage Allowance Payments
            umbrella_fee (Union[Unset, float]): [readonly] Value of any Umbrella fee charged for processing the payment
            app_levy_deduction (Union[Unset, float]): [readonly] Value of any Apprenticeship Levy fee deducted for
                processing the umbrella payment
            payment_after_leaving (Union[Unset, float]): [readonly] Payment After Leaving
            tax_on_payment_after_leaving (Union[Unset, float]): [readonly] Tax On Payment After Leaving
            nil_paid (Union[Unset, int]): [readonly] The number of employees with NilPaid on the PayRun
            leavers (Union[Unset, int]): [readonly] The number of Leavers on ths PayRun
            starters (Union[Unset, int]): [readonly] The number of Starters on this PayRun
            total_cost (Union[Unset, float]):
            recoverable_amounts (Union[Unset, RecoverableAmounts]):
            cost_summary (Union[Unset, PayRunCostSummary]): Represents the various components that make up the costs of a
                PayRun
    """

    basic_pay: Union[Unset, float] = UNSET
    gross: Union[Unset, float] = UNSET
    gross_for_ni: Union[Unset, float] = UNSET
    gross_not_subject_to_employers_ni: Union[Unset, float] = UNSET
    gross_for_tax: Union[Unset, float] = UNSET
    employer_ni: Union[Unset, float] = UNSET
    employee_ni: Union[Unset, float] = UNSET
    employer_ni_off_payroll: Union[Unset, float] = UNSET
    real_time_class_1a_ni: Union[Unset, float] = UNSET
    tax: Union[Unset, float] = UNSET
    net_pay: Union[Unset, float] = UNSET
    adjustments: Union[Unset, float] = UNSET
    additions: Union[Unset, float] = UNSET
    deductions: Union[Unset, float] = UNSET
    take_home_pay: Union[Unset, float] = UNSET
    non_tax_or_nic_pmt: Union[Unset, float] = UNSET
    items_subject_to_class_1nic: Union[Unset, float] = UNSET
    dedns_from_net_pay: Union[Unset, float] = UNSET
    tcp_tcls: Union[Unset, float] = UNSET
    tcp_pp: Union[Unset, float] = UNSET
    tcp_op: Union[Unset, float] = UNSET
    flexi_dd_death: Union[Unset, float] = UNSET
    flexi_dd_death_non_tax: Union[Unset, float] = UNSET
    flexi_dd_pension: Union[Unset, float] = UNSET
    flexi_dd_pension_non_tax: Union[Unset, float] = UNSET
    smp: Union[Unset, float] = UNSET
    spp: Union[Unset, float] = UNSET
    sap: Union[Unset, float] = UNSET
    shpp: Union[Unset, float] = UNSET
    spbp: Union[Unset, float] = UNSET
    ssp: Union[Unset, float] = UNSET
    student_loan_recovered: Union[Unset, float] = UNSET
    postgrad_loan_recovered: Union[Unset, float] = UNSET
    pensionable_earnings: Union[Unset, float] = UNSET
    pensionable_pay: Union[Unset, float] = UNSET
    non_tierable_pay: Union[Unset, float] = UNSET
    employee_pension_contribution: Union[Unset, float] = UNSET
    employee_pension_contribution_avc: Union[Unset, float] = UNSET
    employer_pension_contribution: Union[Unset, float] = UNSET
    empee_pen_contribns_not_paid: Union[Unset, float] = UNSET
    empee_pen_contribns_paid: Union[Unset, float] = UNSET
    attachment_order_deductions: Union[Unset, float] = UNSET
    cis_deduction: Union[Unset, float] = UNSET
    cis_vat: Union[Unset, float] = UNSET
    cis_umbrella_fee: Union[Unset, float] = UNSET
    cis_umbrella_fee_post_tax: Union[Unset, float] = UNSET
    pbik: Union[Unset, float] = UNSET
    maps_miles: Union[Unset, int] = UNSET
    umbrella_fee: Union[Unset, float] = UNSET
    app_levy_deduction: Union[Unset, float] = UNSET
    payment_after_leaving: Union[Unset, float] = UNSET
    tax_on_payment_after_leaving: Union[Unset, float] = UNSET
    nil_paid: Union[Unset, int] = UNSET
    leavers: Union[Unset, int] = UNSET
    starters: Union[Unset, int] = UNSET
    total_cost: Union[Unset, float] = UNSET
    recoverable_amounts: Union[Unset, RecoverableAmounts] = UNSET
    cost_summary: Union[Unset, PayRunCostSummary] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        basic_pay = self.basic_pay
        gross = self.gross
        gross_for_ni = self.gross_for_ni
        gross_not_subject_to_employers_ni = self.gross_not_subject_to_employers_ni
        gross_for_tax = self.gross_for_tax
        employer_ni = self.employer_ni
        employee_ni = self.employee_ni
        employer_ni_off_payroll = self.employer_ni_off_payroll
        real_time_class_1a_ni = self.real_time_class_1a_ni
        tax = self.tax
        net_pay = self.net_pay
        adjustments = self.adjustments
        additions = self.additions
        deductions = self.deductions
        take_home_pay = self.take_home_pay
        non_tax_or_nic_pmt = self.non_tax_or_nic_pmt
        items_subject_to_class_1nic = self.items_subject_to_class_1nic
        dedns_from_net_pay = self.dedns_from_net_pay
        tcp_tcls = self.tcp_tcls
        tcp_pp = self.tcp_pp
        tcp_op = self.tcp_op
        flexi_dd_death = self.flexi_dd_death
        flexi_dd_death_non_tax = self.flexi_dd_death_non_tax
        flexi_dd_pension = self.flexi_dd_pension
        flexi_dd_pension_non_tax = self.flexi_dd_pension_non_tax
        smp = self.smp
        spp = self.spp
        sap = self.sap
        shpp = self.shpp
        spbp = self.spbp
        ssp = self.ssp
        student_loan_recovered = self.student_loan_recovered
        postgrad_loan_recovered = self.postgrad_loan_recovered
        pensionable_earnings = self.pensionable_earnings
        pensionable_pay = self.pensionable_pay
        non_tierable_pay = self.non_tierable_pay
        employee_pension_contribution = self.employee_pension_contribution
        employee_pension_contribution_avc = self.employee_pension_contribution_avc
        employer_pension_contribution = self.employer_pension_contribution
        empee_pen_contribns_not_paid = self.empee_pen_contribns_not_paid
        empee_pen_contribns_paid = self.empee_pen_contribns_paid
        attachment_order_deductions = self.attachment_order_deductions
        cis_deduction = self.cis_deduction
        cis_vat = self.cis_vat
        cis_umbrella_fee = self.cis_umbrella_fee
        cis_umbrella_fee_post_tax = self.cis_umbrella_fee_post_tax
        pbik = self.pbik
        maps_miles = self.maps_miles
        umbrella_fee = self.umbrella_fee
        app_levy_deduction = self.app_levy_deduction
        payment_after_leaving = self.payment_after_leaving
        tax_on_payment_after_leaving = self.tax_on_payment_after_leaving
        nil_paid = self.nil_paid
        leavers = self.leavers
        starters = self.starters
        total_cost = self.total_cost
        recoverable_amounts: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.recoverable_amounts, Unset):
            recoverable_amounts = self.recoverable_amounts.to_dict()

        cost_summary: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.cost_summary, Unset):
            cost_summary = self.cost_summary.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if basic_pay is not UNSET:
            field_dict["basicPay"] = basic_pay
        if gross is not UNSET:
            field_dict["gross"] = gross
        if gross_for_ni is not UNSET:
            field_dict["grossForNi"] = gross_for_ni
        if gross_not_subject_to_employers_ni is not UNSET:
            field_dict[
                "grossNotSubjectToEmployersNi"
            ] = gross_not_subject_to_employers_ni
        if gross_for_tax is not UNSET:
            field_dict["grossForTax"] = gross_for_tax
        if employer_ni is not UNSET:
            field_dict["employerNi"] = employer_ni
        if employee_ni is not UNSET:
            field_dict["employeeNi"] = employee_ni
        if employer_ni_off_payroll is not UNSET:
            field_dict["employerNiOffPayroll"] = employer_ni_off_payroll
        if real_time_class_1a_ni is not UNSET:
            field_dict["realTimeClass1ANi"] = real_time_class_1a_ni
        if tax is not UNSET:
            field_dict["tax"] = tax
        if net_pay is not UNSET:
            field_dict["netPay"] = net_pay
        if adjustments is not UNSET:
            field_dict["adjustments"] = adjustments
        if additions is not UNSET:
            field_dict["additions"] = additions
        if deductions is not UNSET:
            field_dict["deductions"] = deductions
        if take_home_pay is not UNSET:
            field_dict["takeHomePay"] = take_home_pay
        if non_tax_or_nic_pmt is not UNSET:
            field_dict["nonTaxOrNICPmt"] = non_tax_or_nic_pmt
        if items_subject_to_class_1nic is not UNSET:
            field_dict["itemsSubjectToClass1NIC"] = items_subject_to_class_1nic
        if dedns_from_net_pay is not UNSET:
            field_dict["dednsFromNetPay"] = dedns_from_net_pay
        if tcp_tcls is not UNSET:
            field_dict["tcp_Tcls"] = tcp_tcls
        if tcp_pp is not UNSET:
            field_dict["tcp_Pp"] = tcp_pp
        if tcp_op is not UNSET:
            field_dict["tcp_Op"] = tcp_op
        if flexi_dd_death is not UNSET:
            field_dict["flexiDd_Death"] = flexi_dd_death
        if flexi_dd_death_non_tax is not UNSET:
            field_dict["flexiDd_Death_NonTax"] = flexi_dd_death_non_tax
        if flexi_dd_pension is not UNSET:
            field_dict["flexiDd_Pension"] = flexi_dd_pension
        if flexi_dd_pension_non_tax is not UNSET:
            field_dict["flexiDd_Pension_NonTax"] = flexi_dd_pension_non_tax
        if smp is not UNSET:
            field_dict["smp"] = smp
        if spp is not UNSET:
            field_dict["spp"] = spp
        if sap is not UNSET:
            field_dict["sap"] = sap
        if shpp is not UNSET:
            field_dict["shpp"] = shpp
        if spbp is not UNSET:
            field_dict["spbp"] = spbp
        if ssp is not UNSET:
            field_dict["ssp"] = ssp
        if student_loan_recovered is not UNSET:
            field_dict["studentLoanRecovered"] = student_loan_recovered
        if postgrad_loan_recovered is not UNSET:
            field_dict["postgradLoanRecovered"] = postgrad_loan_recovered
        if pensionable_earnings is not UNSET:
            field_dict["pensionableEarnings"] = pensionable_earnings
        if pensionable_pay is not UNSET:
            field_dict["pensionablePay"] = pensionable_pay
        if non_tierable_pay is not UNSET:
            field_dict["nonTierablePay"] = non_tierable_pay
        if employee_pension_contribution is not UNSET:
            field_dict["employeePensionContribution"] = employee_pension_contribution
        if employee_pension_contribution_avc is not UNSET:
            field_dict[
                "employeePensionContributionAvc"
            ] = employee_pension_contribution_avc
        if employer_pension_contribution is not UNSET:
            field_dict["employerPensionContribution"] = employer_pension_contribution
        if empee_pen_contribns_not_paid is not UNSET:
            field_dict["empeePenContribnsNotPaid"] = empee_pen_contribns_not_paid
        if empee_pen_contribns_paid is not UNSET:
            field_dict["empeePenContribnsPaid"] = empee_pen_contribns_paid
        if attachment_order_deductions is not UNSET:
            field_dict["attachmentOrderDeductions"] = attachment_order_deductions
        if cis_deduction is not UNSET:
            field_dict["cisDeduction"] = cis_deduction
        if cis_vat is not UNSET:
            field_dict["cisVat"] = cis_vat
        if cis_umbrella_fee is not UNSET:
            field_dict["cisUmbrellaFee"] = cis_umbrella_fee
        if cis_umbrella_fee_post_tax is not UNSET:
            field_dict["cisUmbrellaFeePostTax"] = cis_umbrella_fee_post_tax
        if pbik is not UNSET:
            field_dict["pbik"] = pbik
        if maps_miles is not UNSET:
            field_dict["mapsMiles"] = maps_miles
        if umbrella_fee is not UNSET:
            field_dict["umbrellaFee"] = umbrella_fee
        if app_levy_deduction is not UNSET:
            field_dict["appLevyDeduction"] = app_levy_deduction
        if payment_after_leaving is not UNSET:
            field_dict["paymentAfterLeaving"] = payment_after_leaving
        if tax_on_payment_after_leaving is not UNSET:
            field_dict["taxOnPaymentAfterLeaving"] = tax_on_payment_after_leaving
        if nil_paid is not UNSET:
            field_dict["nilPaid"] = nil_paid
        if leavers is not UNSET:
            field_dict["leavers"] = leavers
        if starters is not UNSET:
            field_dict["starters"] = starters
        if total_cost is not UNSET:
            field_dict["totalCost"] = total_cost
        if recoverable_amounts is not UNSET:
            field_dict["recoverableAmounts"] = recoverable_amounts
        if cost_summary is not UNSET:
            field_dict["costSummary"] = cost_summary

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        basic_pay = d.pop("basicPay", UNSET)

        gross = d.pop("gross", UNSET)

        gross_for_ni = d.pop("grossForNi", UNSET)

        gross_not_subject_to_employers_ni = d.pop("grossNotSubjectToEmployersNi", UNSET)

        gross_for_tax = d.pop("grossForTax", UNSET)

        employer_ni = d.pop("employerNi", UNSET)

        employee_ni = d.pop("employeeNi", UNSET)

        employer_ni_off_payroll = d.pop("employerNiOffPayroll", UNSET)

        real_time_class_1a_ni = d.pop("realTimeClass1ANi", UNSET)

        tax = d.pop("tax", UNSET)

        net_pay = d.pop("netPay", UNSET)

        adjustments = d.pop("adjustments", UNSET)

        additions = d.pop("additions", UNSET)

        deductions = d.pop("deductions", UNSET)

        take_home_pay = d.pop("takeHomePay", UNSET)

        non_tax_or_nic_pmt = d.pop("nonTaxOrNICPmt", UNSET)

        items_subject_to_class_1nic = d.pop("itemsSubjectToClass1NIC", UNSET)

        dedns_from_net_pay = d.pop("dednsFromNetPay", UNSET)

        tcp_tcls = d.pop("tcp_Tcls", UNSET)

        tcp_pp = d.pop("tcp_Pp", UNSET)

        tcp_op = d.pop("tcp_Op", UNSET)

        flexi_dd_death = d.pop("flexiDd_Death", UNSET)

        flexi_dd_death_non_tax = d.pop("flexiDd_Death_NonTax", UNSET)

        flexi_dd_pension = d.pop("flexiDd_Pension", UNSET)

        flexi_dd_pension_non_tax = d.pop("flexiDd_Pension_NonTax", UNSET)

        smp = d.pop("smp", UNSET)

        spp = d.pop("spp", UNSET)

        sap = d.pop("sap", UNSET)

        shpp = d.pop("shpp", UNSET)

        spbp = d.pop("spbp", UNSET)

        ssp = d.pop("ssp", UNSET)

        student_loan_recovered = d.pop("studentLoanRecovered", UNSET)

        postgrad_loan_recovered = d.pop("postgradLoanRecovered", UNSET)

        pensionable_earnings = d.pop("pensionableEarnings", UNSET)

        pensionable_pay = d.pop("pensionablePay", UNSET)

        non_tierable_pay = d.pop("nonTierablePay", UNSET)

        employee_pension_contribution = d.pop("employeePensionContribution", UNSET)

        employee_pension_contribution_avc = d.pop(
            "employeePensionContributionAvc", UNSET
        )

        employer_pension_contribution = d.pop("employerPensionContribution", UNSET)

        empee_pen_contribns_not_paid = d.pop("empeePenContribnsNotPaid", UNSET)

        empee_pen_contribns_paid = d.pop("empeePenContribnsPaid", UNSET)

        attachment_order_deductions = d.pop("attachmentOrderDeductions", UNSET)

        cis_deduction = d.pop("cisDeduction", UNSET)

        cis_vat = d.pop("cisVat", UNSET)

        cis_umbrella_fee = d.pop("cisUmbrellaFee", UNSET)

        cis_umbrella_fee_post_tax = d.pop("cisUmbrellaFeePostTax", UNSET)

        pbik = d.pop("pbik", UNSET)

        maps_miles = d.pop("mapsMiles", UNSET)

        umbrella_fee = d.pop("umbrellaFee", UNSET)

        app_levy_deduction = d.pop("appLevyDeduction", UNSET)

        payment_after_leaving = d.pop("paymentAfterLeaving", UNSET)

        tax_on_payment_after_leaving = d.pop("taxOnPaymentAfterLeaving", UNSET)

        nil_paid = d.pop("nilPaid", UNSET)

        leavers = d.pop("leavers", UNSET)

        starters = d.pop("starters", UNSET)

        total_cost = d.pop("totalCost", UNSET)

        _recoverable_amounts = d.pop("recoverableAmounts", UNSET)
        recoverable_amounts: Union[Unset, RecoverableAmounts]
        if isinstance(_recoverable_amounts, Unset):
            recoverable_amounts = UNSET
        else:
            recoverable_amounts = RecoverableAmounts.from_dict(_recoverable_amounts)

        _cost_summary = d.pop("costSummary", UNSET)
        cost_summary: Union[Unset, PayRunCostSummary]
        if isinstance(_cost_summary, Unset):
            cost_summary = UNSET
        else:
            cost_summary = PayRunCostSummary.from_dict(_cost_summary)

        pay_run_totals = cls(
            basic_pay=basic_pay,
            gross=gross,
            gross_for_ni=gross_for_ni,
            gross_not_subject_to_employers_ni=gross_not_subject_to_employers_ni,
            gross_for_tax=gross_for_tax,
            employer_ni=employer_ni,
            employee_ni=employee_ni,
            employer_ni_off_payroll=employer_ni_off_payroll,
            real_time_class_1a_ni=real_time_class_1a_ni,
            tax=tax,
            net_pay=net_pay,
            adjustments=adjustments,
            additions=additions,
            deductions=deductions,
            take_home_pay=take_home_pay,
            non_tax_or_nic_pmt=non_tax_or_nic_pmt,
            items_subject_to_class_1nic=items_subject_to_class_1nic,
            dedns_from_net_pay=dedns_from_net_pay,
            tcp_tcls=tcp_tcls,
            tcp_pp=tcp_pp,
            tcp_op=tcp_op,
            flexi_dd_death=flexi_dd_death,
            flexi_dd_death_non_tax=flexi_dd_death_non_tax,
            flexi_dd_pension=flexi_dd_pension,
            flexi_dd_pension_non_tax=flexi_dd_pension_non_tax,
            smp=smp,
            spp=spp,
            sap=sap,
            shpp=shpp,
            spbp=spbp,
            ssp=ssp,
            student_loan_recovered=student_loan_recovered,
            postgrad_loan_recovered=postgrad_loan_recovered,
            pensionable_earnings=pensionable_earnings,
            pensionable_pay=pensionable_pay,
            non_tierable_pay=non_tierable_pay,
            employee_pension_contribution=employee_pension_contribution,
            employee_pension_contribution_avc=employee_pension_contribution_avc,
            employer_pension_contribution=employer_pension_contribution,
            empee_pen_contribns_not_paid=empee_pen_contribns_not_paid,
            empee_pen_contribns_paid=empee_pen_contribns_paid,
            attachment_order_deductions=attachment_order_deductions,
            cis_deduction=cis_deduction,
            cis_vat=cis_vat,
            cis_umbrella_fee=cis_umbrella_fee,
            cis_umbrella_fee_post_tax=cis_umbrella_fee_post_tax,
            pbik=pbik,
            maps_miles=maps_miles,
            umbrella_fee=umbrella_fee,
            app_levy_deduction=app_levy_deduction,
            payment_after_leaving=payment_after_leaving,
            tax_on_payment_after_leaving=tax_on_payment_after_leaving,
            nil_paid=nil_paid,
            leavers=leavers,
            starters=starters,
            total_cost=total_cost,
            recoverable_amounts=recoverable_amounts,
            cost_summary=cost_summary,
        )

        return pay_run_totals



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.bank_details import BankDetails
from ..types import UNSET, Unset

T = TypeVar("T", bound="Payee")


@attr.s(auto_attribs=True)
class Payee:
    """
    Attributes:
        title (str): The name of this Payee
        bank_details (Union[Unset, BankDetails]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    title: str
    bank_details: Union[Unset, BankDetails] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        title = self.title
        bank_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bank_details, Unset):
            bank_details = self.bank_details.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "title": title,
            }
        )
        if bank_details is not UNSET:
            field_dict["bankDetails"] = bank_details
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        title = d.pop("title")

        _bank_details = d.pop("bankDetails", UNSET)
        bank_details: Union[Unset, BankDetails]
        if isinstance(_bank_details, Unset):
            bank_details = UNSET
        else:
            bank_details = BankDetails.from_dict(_bank_details)

        id = d.pop("id", UNSET)

        payee = cls(
            title=title,
            bank_details=bank_details,
            id=id,
        )

        return payee



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.payslip_line_type import PayslipLineType
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayslipLine")


@attr.s(auto_attribs=True)
class PayslipLine:
    """
    Attributes:
        type (Union[Unset, PayslipLineType]):
        code (Union[Unset, None, str]):
        description (Union[Unset, None, str]):
        secondary_description (Union[Unset, None, str]):
        value (Union[Unset, float]):
        multiplier (Union[Unset, None, float]): [readonly] If the Code is a multiplier code then this is the multiplier.
            Value has already been calculated so this is just for reference
        attachment_order_id (Union[Unset, None, str]): [readonly] The Id of the associated AttachmentOrder. Only
            included if the Code is AEO
        pension_id (Union[Unset, None, str]): [readonly] The Id of the associated Pension. Only included if the Code is
            PENSION, PENSIONSS or PENSIONRAS
        loan_id (Union[Unset, None, str]): [readonly] The Id of the associated Loan, if any.
        currency_symbol (Union[Unset, None, str]):
        formatted_value (Union[Unset, None, str]):
        is_net_to_gross (Union[Unset, bool]): If the PayLine is a fixed ammount addition without multiplier then this
            property may be set to true so that the amount of the addition to be considered a take home pay target.
        target_net_to_gross_value (Union[Unset, None, float]): The orginal net fixed addition amount that is considered
            to be a take home pay target.
        has_secondary_description (Union[Unset, bool]):
    """

    type: Union[Unset, PayslipLineType] = UNSET
    code: Union[Unset, None, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    secondary_description: Union[Unset, None, str] = UNSET
    value: Union[Unset, float] = UNSET
    multiplier: Union[Unset, None, float] = UNSET
    attachment_order_id: Union[Unset, None, str] = UNSET
    pension_id: Union[Unset, None, str] = UNSET
    loan_id: Union[Unset, None, str] = UNSET
    currency_symbol: Union[Unset, None, str] = UNSET
    formatted_value: Union[Unset, None, str] = UNSET
    is_net_to_gross: Union[Unset, bool] = UNSET
    target_net_to_gross_value: Union[Unset, None, float] = UNSET
    has_secondary_description: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        code = self.code
        description = self.description
        secondary_description = self.secondary_description
        value = self.value
        multiplier = self.multiplier
        attachment_order_id = self.attachment_order_id
        pension_id = self.pension_id
        loan_id = self.loan_id
        currency_symbol = self.currency_symbol
        formatted_value = self.formatted_value
        is_net_to_gross = self.is_net_to_gross
        target_net_to_gross_value = self.target_net_to_gross_value
        has_secondary_description = self.has_secondary_description

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if code is not UNSET:
            field_dict["code"] = code
        if description is not UNSET:
            field_dict["description"] = description
        if secondary_description is not UNSET:
            field_dict["secondaryDescription"] = secondary_description
        if value is not UNSET:
            field_dict["value"] = value
        if multiplier is not UNSET:
            field_dict["multiplier"] = multiplier
        if attachment_order_id is not UNSET:
            field_dict["attachmentOrderId"] = attachment_order_id
        if pension_id is not UNSET:
            field_dict["pensionId"] = pension_id
        if loan_id is not UNSET:
            field_dict["loanId"] = loan_id
        if currency_symbol is not UNSET:
            field_dict["currencySymbol"] = currency_symbol
        if formatted_value is not UNSET:
            field_dict["formattedValue"] = formatted_value
        if is_net_to_gross is not UNSET:
            field_dict["isNetToGross"] = is_net_to_gross
        if target_net_to_gross_value is not UNSET:
            field_dict["targetNetToGrossValue"] = target_net_to_gross_value
        if has_secondary_description is not UNSET:
            field_dict["hasSecondaryDescription"] = has_secondary_description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, PayslipLineType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = PayslipLineType(_type)

        code = d.pop("code", UNSET)

        description = d.pop("description", UNSET)

        secondary_description = d.pop("secondaryDescription", UNSET)

        value = d.pop("value", UNSET)

        multiplier = d.pop("multiplier", UNSET)

        attachment_order_id = d.pop("attachmentOrderId", UNSET)

        pension_id = d.pop("pensionId", UNSET)

        loan_id = d.pop("loanId", UNSET)

        currency_symbol = d.pop("currencySymbol", UNSET)

        formatted_value = d.pop("formattedValue", UNSET)

        is_net_to_gross = d.pop("isNetToGross", UNSET)

        target_net_to_gross_value = d.pop("targetNetToGrossValue", UNSET)

        has_secondary_description = d.pop("hasSecondaryDescription", UNSET)

        payslip_line = cls(
            type=type,
            code=code,
            description=description,
            secondary_description=secondary_description,
            value=value,
            multiplier=multiplier,
            attachment_order_id=attachment_order_id,
            pension_id=pension_id,
            loan_id=loan_id,
            currency_symbol=currency_symbol,
            formatted_value=formatted_value,
            is_net_to_gross=is_net_to_gross,
            target_net_to_gross_value=target_net_to_gross_value,
            has_secondary_description=has_secondary_description,
        )

        return payslip_line



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.full_summary_of_pay_report_line import FullSummaryOfPayReportLine
from ..models.item import Item
from ..models.pay_periods import PayPeriods
from ..models.report import Report
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="FullSummaryOfPayReport")


@attr.s(auto_attribs=True)
class FullSummaryOfPayReport:
    """
    Attributes:
        lines (Union[Unset, None, List[FullSummaryOfPayReportLine]]):
        deduction_lines (Union[Unset, None, List[FullSummaryOfPayReportLine]]):
        employer (Union[Unset, Item]):
        pay_period (Union[Unset, PayPeriods]):
        ordinal (Union[Unset, int]):
        period (Union[Unset, int]):
        period_to (Union[Unset, int]):
        start_period_name (Union[Unset, None, str]):
        end_period_name (Union[Unset, None, str]):
        start_date (Union[Unset, datetime.date]):
        end_date (Union[Unset, datetime.date]):
        report (Union[Unset, Report]):
        tax_year (Union[Unset, TaxYear]):
    """

    lines: Union[Unset, None, List[FullSummaryOfPayReportLine]] = UNSET
    deduction_lines: Union[Unset, None, List[FullSummaryOfPayReportLine]] = UNSET
    employer: Union[Unset, Item] = UNSET
    pay_period: Union[Unset, PayPeriods] = UNSET
    ordinal: Union[Unset, int] = UNSET
    period: Union[Unset, int] = UNSET
    period_to: Union[Unset, int] = UNSET
    start_period_name: Union[Unset, None, str] = UNSET
    end_period_name: Union[Unset, None, str] = UNSET
    start_date: Union[Unset, datetime.date] = UNSET
    end_date: Union[Unset, datetime.date] = UNSET
    report: Union[Unset, Report] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.lines, Unset):
            if self.lines is None:
                lines = None
            else:
                lines = []
                for lines_item_data in self.lines:
                    lines_item = lines_item_data.to_dict()

                    lines.append(lines_item)

        deduction_lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.deduction_lines, Unset):
            if self.deduction_lines is None:
                deduction_lines = None
            else:
                deduction_lines = []
                for deduction_lines_item_data in self.deduction_lines:
                    deduction_lines_item = deduction_lines_item_data.to_dict()

                    deduction_lines.append(deduction_lines_item)

        employer: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer, Unset):
            employer = self.employer.to_dict()

        pay_period: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period, Unset):
            pay_period = self.pay_period.value

        ordinal = self.ordinal
        period = self.period
        period_to = self.period_to
        start_period_name = self.start_period_name
        end_period_name = self.end_period_name
        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        report: Union[Unset, str] = UNSET
        if not isinstance(self.report, Unset):
            report = self.report.value

        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if lines is not UNSET:
            field_dict["lines"] = lines
        if deduction_lines is not UNSET:
            field_dict["deductionLines"] = deduction_lines
        if employer is not UNSET:
            field_dict["employer"] = employer
        if pay_period is not UNSET:
            field_dict["payPeriod"] = pay_period
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if period is not UNSET:
            field_dict["period"] = period
        if period_to is not UNSET:
            field_dict["periodTo"] = period_to
        if start_period_name is not UNSET:
            field_dict["startPeriodName"] = start_period_name
        if end_period_name is not UNSET:
            field_dict["endPeriodName"] = end_period_name
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if report is not UNSET:
            field_dict["report"] = report
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        lines = []
        _lines = d.pop("lines", UNSET)
        for lines_item_data in _lines or []:
            lines_item = FullSummaryOfPayReportLine.from_dict(lines_item_data)

            lines.append(lines_item)

        deduction_lines = []
        _deduction_lines = d.pop("deductionLines", UNSET)
        for deduction_lines_item_data in _deduction_lines or []:
            deduction_lines_item = FullSummaryOfPayReportLine.from_dict(
                deduction_lines_item_data
            )

            deduction_lines.append(deduction_lines_item)

        _employer = d.pop("employer", UNSET)
        employer: Union[Unset, Item]
        if isinstance(_employer, Unset):
            employer = UNSET
        else:
            employer = Item.from_dict(_employer)

        _pay_period = d.pop("payPeriod", UNSET)
        pay_period: Union[Unset, PayPeriods]
        if isinstance(_pay_period, Unset):
            pay_period = UNSET
        else:
            pay_period = PayPeriods(_pay_period)

        ordinal = d.pop("ordinal", UNSET)

        period = d.pop("period", UNSET)

        period_to = d.pop("periodTo", UNSET)

        start_period_name = d.pop("startPeriodName", UNSET)

        end_period_name = d.pop("endPeriodName", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.date]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date).date()

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.date]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date).date()

        _report = d.pop("report", UNSET)
        report: Union[Unset, Report]
        if isinstance(_report, Unset):
            report = UNSET
        else:
            report = Report(_report)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        full_summary_of_pay_report = cls(
            lines=lines,
            deduction_lines=deduction_lines,
            employer=employer,
            pay_period=pay_period,
            ordinal=ordinal,
            period=period,
            period_to=period_to,
            start_period_name=start_period_name,
            end_period_name=end_period_name,
            start_date=start_date,
            end_date=end_date,
            report=report,
            tax_year=tax_year,
        )

        return full_summary_of_pay_report



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.pay_code import PayCode
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayRunSummaryLine")


@attr.s(auto_attribs=True)
class PayRunSummaryLine:
    """An array of this model is used to represent a summary of a PayRun.
    This would typically be used for creating accounting entries for the PayRun.

        Attributes:
            nominal_code (Union[Unset, None, str]): [readonly] If you've set up NominalCodeMapping then the relevant Nominal
                code for the PayCode will be shown here.
            department_code (Union[Unset, None, str]): [readonly] If the journal is split by department then the relevant
                department code is shown here.
            nominal_name (Union[Unset, None, str]): [readonly] As above
            description (Union[Unset, None, str]): [readonly] A description of what this line summarises.
            qty (Union[Unset, None, float]): [readonly] If the PayCode is a Multiplier code then this will contain the
                number of days/hours
            value (Union[Unset, float]): [readonly] The total value for this line.
            pay_code (Union[Unset, PayCode]): Each PayLine has a Code. The Code will match the Code property of a PayCode.
                The PayCode that is used determines how the amount is treated with regards to tax, NI and pensions
    """

    nominal_code: Union[Unset, None, str] = UNSET
    department_code: Union[Unset, None, str] = UNSET
    nominal_name: Union[Unset, None, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    qty: Union[Unset, None, float] = UNSET
    value: Union[Unset, float] = UNSET
    pay_code: Union[Unset, PayCode] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        nominal_code = self.nominal_code
        department_code = self.department_code
        nominal_name = self.nominal_name
        description = self.description
        qty = self.qty
        value = self.value
        pay_code: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pay_code, Unset):
            pay_code = self.pay_code.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if nominal_code is not UNSET:
            field_dict["nominalCode"] = nominal_code
        if department_code is not UNSET:
            field_dict["departmentCode"] = department_code
        if nominal_name is not UNSET:
            field_dict["nominalName"] = nominal_name
        if description is not UNSET:
            field_dict["description"] = description
        if qty is not UNSET:
            field_dict["qty"] = qty
        if value is not UNSET:
            field_dict["value"] = value
        if pay_code is not UNSET:
            field_dict["payCode"] = pay_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        nominal_code = d.pop("nominalCode", UNSET)

        department_code = d.pop("departmentCode", UNSET)

        nominal_name = d.pop("nominalName", UNSET)

        description = d.pop("description", UNSET)

        qty = d.pop("qty", UNSET)

        value = d.pop("value", UNSET)

        _pay_code = d.pop("payCode", UNSET)
        pay_code: Union[Unset, PayCode]
        if isinstance(_pay_code, Unset):
            pay_code = UNSET
        else:
            pay_code = PayCode.from_dict(_pay_code)

        pay_run_summary_line = cls(
            nominal_code=nominal_code,
            department_code=department_code,
            nominal_name=nominal_name,
            description=description,
            qty=qty,
            value=value,
            pay_code=pay_code,
        )

        return pay_run_summary_line



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.auto_pilot_finalise_time import AutoPilotFinaliseTime
from ..models.payrun_email import PayrunEmail
from ..types import UNSET, Unset

T = TypeVar("T", bound="AutomationSettings")


@attr.s(auto_attribs=True)
class AutomationSettings:
    """Configures various automation settings for an Employer

    Attributes:
        enable_auto_pilot (Union[Unset, bool]): IF enabled then payruns will be automatically finalised on the payment
            date and the next payrun will be started
        auto_pilot_time (Union[Unset, AutoPilotFinaliseTime]):
        auto_pilot_offset (Union[Unset, int]): How many days before the payment date a payrun should be finalised.
            Set it to 0 if you want the payrun to be automatically finalised on the payment date itself
        enable_auto_open (Union[Unset, bool]): IF enabled then whenever you close a payrun, the next one will be
            started. Automatically set to true if EnableAutoPilot is true
        auto_submit_fps (Union[Unset, bool]): If set to true, we'll automatically send your FPS to HMRC whenever you
            finalise a PayRun.
            This property will always have the same value as the property with the same name on the RtiSubmissionSettings
            model.
        auto_submit_payments (Union[Unset, bool]): If set to true, we'll automatically submit payments whenever you
            finalise a PayRun.
            The employer must be connected to an ExternalDataProvider supporting Type of 'Payments'.
        auto_submit_journal (Union[Unset, bool]): If set to true, we'll automatically submit the payroll journal
            whenever you finalise a PayRun.
            The employer must be connected to an ExternalDataProvider supporting Type of 'Accounting'.
        auto_submit_payments_employee (Union[Unset, bool]): Used in conjunction with AutoSubmitPayments.
        auto_submit_payments_hmrc (Union[Unset, bool]): Used in conjunction with AutoSubmitPayments.
        auto_submit_payments_deductions (Union[Unset, bool]): Used in conjunction with AutoSubmitPayments.
        auto_submit_payments_aeos (Union[Unset, bool]): Used in conjunction with AutoSubmitPayments.
        auto_submit_payments_pensions (Union[Unset, bool]): Used in conjunction with AutoSubmitPayments.
        employees_without_email_address (Union[Unset, int]): [readonly] A count of how many employees or subcontractors
            for this employer do not have email addresses
        employees_with_email_address_but_not_auto_sending (Union[Unset, int]): [readonly] A count of how many employees
            or subcontractors for this employer do have email addresses but don't have the option enabled to auto-email
            payslips
        subcontractors_not_auto_sending_statement (Union[Unset, int]): [readonly] A count of how many subscontractors
            for this employer do don't have the option enabled to auto-email statement
        coding_notices_automatically_applied (Union[Unset, bool]): [readonly] An indicator of whether or not this
            employer is automatically applying DpsNotices
        payrun_emails (Union[Unset, None, List[PayrunEmail]]): Automated emails that will be sent when a PayRun is
            finalised
    """

    enable_auto_pilot: Union[Unset, bool] = UNSET
    auto_pilot_time: Union[Unset, AutoPilotFinaliseTime] = UNSET
    auto_pilot_offset: Union[Unset, int] = UNSET
    enable_auto_open: Union[Unset, bool] = UNSET
    auto_submit_fps: Union[Unset, bool] = UNSET
    auto_submit_payments: Union[Unset, bool] = UNSET
    auto_submit_journal: Union[Unset, bool] = UNSET
    auto_submit_payments_employee: Union[Unset, bool] = UNSET
    auto_submit_payments_hmrc: Union[Unset, bool] = UNSET
    auto_submit_payments_deductions: Union[Unset, bool] = UNSET
    auto_submit_payments_aeos: Union[Unset, bool] = UNSET
    auto_submit_payments_pensions: Union[Unset, bool] = UNSET
    employees_without_email_address: Union[Unset, int] = UNSET
    employees_with_email_address_but_not_auto_sending: Union[Unset, int] = UNSET
    subcontractors_not_auto_sending_statement: Union[Unset, int] = UNSET
    coding_notices_automatically_applied: Union[Unset, bool] = UNSET
    payrun_emails: Union[Unset, None, List[PayrunEmail]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        enable_auto_pilot = self.enable_auto_pilot
        auto_pilot_time: Union[Unset, str] = UNSET
        if not isinstance(self.auto_pilot_time, Unset):
            auto_pilot_time = self.auto_pilot_time.value

        auto_pilot_offset = self.auto_pilot_offset
        enable_auto_open = self.enable_auto_open
        auto_submit_fps = self.auto_submit_fps
        auto_submit_payments = self.auto_submit_payments
        auto_submit_journal = self.auto_submit_journal
        auto_submit_payments_employee = self.auto_submit_payments_employee
        auto_submit_payments_hmrc = self.auto_submit_payments_hmrc
        auto_submit_payments_deductions = self.auto_submit_payments_deductions
        auto_submit_payments_aeos = self.auto_submit_payments_aeos
        auto_submit_payments_pensions = self.auto_submit_payments_pensions
        employees_without_email_address = self.employees_without_email_address
        employees_with_email_address_but_not_auto_sending = (
            self.employees_with_email_address_but_not_auto_sending
        )
        subcontractors_not_auto_sending_statement = (
            self.subcontractors_not_auto_sending_statement
        )
        coding_notices_automatically_applied = self.coding_notices_automatically_applied
        payrun_emails: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.payrun_emails, Unset):
            if self.payrun_emails is None:
                payrun_emails = None
            else:
                payrun_emails = []
                for payrun_emails_item_data in self.payrun_emails:
                    payrun_emails_item = payrun_emails_item_data.to_dict()

                    payrun_emails.append(payrun_emails_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if enable_auto_pilot is not UNSET:
            field_dict["enableAutoPilot"] = enable_auto_pilot
        if auto_pilot_time is not UNSET:
            field_dict["autoPilotTime"] = auto_pilot_time
        if auto_pilot_offset is not UNSET:
            field_dict["autoPilotOffset"] = auto_pilot_offset
        if enable_auto_open is not UNSET:
            field_dict["enableAutoOpen"] = enable_auto_open
        if auto_submit_fps is not UNSET:
            field_dict["autoSubmitFps"] = auto_submit_fps
        if auto_submit_payments is not UNSET:
            field_dict["autoSubmitPayments"] = auto_submit_payments
        if auto_submit_journal is not UNSET:
            field_dict["autoSubmitJournal"] = auto_submit_journal
        if auto_submit_payments_employee is not UNSET:
            field_dict["autoSubmitPayments_Employee"] = auto_submit_payments_employee
        if auto_submit_payments_hmrc is not UNSET:
            field_dict["autoSubmitPayments_Hmrc"] = auto_submit_payments_hmrc
        if auto_submit_payments_deductions is not UNSET:
            field_dict[
                "autoSubmitPayments_Deductions"
            ] = auto_submit_payments_deductions
        if auto_submit_payments_aeos is not UNSET:
            field_dict["autoSubmitPayments_Aeos"] = auto_submit_payments_aeos
        if auto_submit_payments_pensions is not UNSET:
            field_dict["autoSubmitPayments_Pensions"] = auto_submit_payments_pensions
        if employees_without_email_address is not UNSET:
            field_dict["employeesWithoutEmailAddress"] = employees_without_email_address
        if employees_with_email_address_but_not_auto_sending is not UNSET:
            field_dict[
                "employeesWithEmailAddressButNotAutoSending"
            ] = employees_with_email_address_but_not_auto_sending
        if subcontractors_not_auto_sending_statement is not UNSET:
            field_dict[
                "subcontractorsNotAutoSendingStatement"
            ] = subcontractors_not_auto_sending_statement
        if coding_notices_automatically_applied is not UNSET:
            field_dict[
                "codingNoticesAutomaticallyApplied"
            ] = coding_notices_automatically_applied
        if payrun_emails is not UNSET:
            field_dict["payrunEmails"] = payrun_emails

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enable_auto_pilot = d.pop("enableAutoPilot", UNSET)

        _auto_pilot_time = d.pop("autoPilotTime", UNSET)
        auto_pilot_time: Union[Unset, AutoPilotFinaliseTime]
        if isinstance(_auto_pilot_time, Unset):
            auto_pilot_time = UNSET
        else:
            auto_pilot_time = AutoPilotFinaliseTime(_auto_pilot_time)

        auto_pilot_offset = d.pop("autoPilotOffset", UNSET)

        enable_auto_open = d.pop("enableAutoOpen", UNSET)

        auto_submit_fps = d.pop("autoSubmitFps", UNSET)

        auto_submit_payments = d.pop("autoSubmitPayments", UNSET)

        auto_submit_journal = d.pop("autoSubmitJournal", UNSET)

        auto_submit_payments_employee = d.pop("autoSubmitPayments_Employee", UNSET)

        auto_submit_payments_hmrc = d.pop("autoSubmitPayments_Hmrc", UNSET)

        auto_submit_payments_deductions = d.pop("autoSubmitPayments_Deductions", UNSET)

        auto_submit_payments_aeos = d.pop("autoSubmitPayments_Aeos", UNSET)

        auto_submit_payments_pensions = d.pop("autoSubmitPayments_Pensions", UNSET)

        employees_without_email_address = d.pop("employeesWithoutEmailAddress", UNSET)

        employees_with_email_address_but_not_auto_sending = d.pop(
            "employeesWithEmailAddressButNotAutoSending", UNSET
        )

        subcontractors_not_auto_sending_statement = d.pop(
            "subcontractorsNotAutoSendingStatement", UNSET
        )

        coding_notices_automatically_applied = d.pop(
            "codingNoticesAutomaticallyApplied", UNSET
        )

        payrun_emails = []
        _payrun_emails = d.pop("payrunEmails", UNSET)
        for payrun_emails_item_data in _payrun_emails or []:
            payrun_emails_item = PayrunEmail.from_dict(payrun_emails_item_data)

            payrun_emails.append(payrun_emails_item)

        automation_settings = cls(
            enable_auto_pilot=enable_auto_pilot,
            auto_pilot_time=auto_pilot_time,
            auto_pilot_offset=auto_pilot_offset,
            enable_auto_open=enable_auto_open,
            auto_submit_fps=auto_submit_fps,
            auto_submit_payments=auto_submit_payments,
            auto_submit_journal=auto_submit_journal,
            auto_submit_payments_employee=auto_submit_payments_employee,
            auto_submit_payments_hmrc=auto_submit_payments_hmrc,
            auto_submit_payments_deductions=auto_submit_payments_deductions,
            auto_submit_payments_aeos=auto_submit_payments_aeos,
            auto_submit_payments_pensions=auto_submit_payments_pensions,
            employees_without_email_address=employees_without_email_address,
            employees_with_email_address_but_not_auto_sending=employees_with_email_address_but_not_auto_sending,
            subcontractors_not_auto_sending_statement=subcontractors_not_auto_sending_statement,
            coding_notices_automatically_applied=coding_notices_automatically_applied,
            payrun_emails=payrun_emails,
        )

        return automation_settings



from enum import Enum


class RtiSenderType(str, Enum):
    ACTINGINCAPACITY = "ActingInCapacity"
    AGENT = "Agent"
    BUREAU = "Bureau"
    COMPANY = "Company"
    EMPLOYER = "Employer"
    GOVERNMENT = "Government"
    INDIVIDUAL = "Individual"
    OTHER = "Other"
    PARTNERSHIP = "Partnership"
    TRUST = "Trust"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.p11_detailed_ni_values import P11DetailedNiValues
from ..models.p11_ni_and_stat_payments_line import P11NiAndStatPaymentsLine
from ..models.p11_ni_and_stat_payments_totals_line import P11NiAndStatPaymentsTotalsLine
from ..models.p11_paye_line import P11PayeLine
from ..models.p11_paye_summary import P11PayeSummary
from ..models.p11_paye_totals_line import P11PayeTotalsLine
from ..models.report import Report
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11Detailed")


@attr.s(auto_attribs=True)
class P11Detailed:
    """A more comprehensive P11 Report.
    Our Reports API can return this to you in several formats including as a PDF file
    If you request a JSON object then it is represented using this model.

        Attributes:
            employer_name (Union[Unset, None, str]): [readonly]
            payroll_code (Union[Unset, None, str]): [readonly]
            employer_office_no (Union[Unset, None, str]): [readonly]
            employer_paye_ref (Union[Unset, None, str]): [readonly]
            firstname (Union[Unset, None, str]): [readonly]
            middlename (Union[Unset, None, str]): [readonly]
            surname (Union[Unset, None, str]): [readonly]
            ni_number (Union[Unset, None, str]): [readonly]
            tax_code (Union[Unset, None, str]): [readonly]
            date_of_birth (Union[Unset, datetime.date]): [readonly]
            join_date (Union[Unset, datetime.date]): [readonly]
            leave_date (Union[Unset, None, datetime.date]): [readonly]
            is_director (Union[Unset, bool]): [readonly]
            ni_and_stat_payments_lines (Union[Unset, None, List[P11NiAndStatPaymentsLine]]): [readonly]
            totals_line (Union[Unset, P11NiAndStatPaymentsTotalsLine]): Summary line for the NI Contributions and Statutory
                Payments table in the P11 Detailed report
            ni_summary (Union[Unset, None, List[P11DetailedNiValues]]): [readonly]
            paye_lines (Union[Unset, None, List[P11PayeLine]]): [readonly]
            paye_totals_line (Union[Unset, P11PayeTotalsLine]): Summary line for the Paye Income Tax table in the P11
                Detailed report
            paye_summary (Union[Unset, P11PayeSummary]): Forms the PAYE summary in the P11 Detailed report
            report (Union[Unset, Report]):
            tax_year (Union[Unset, TaxYear]):
    """

    employer_name: Union[Unset, None, str] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    employer_office_no: Union[Unset, None, str] = UNSET
    employer_paye_ref: Union[Unset, None, str] = UNSET
    firstname: Union[Unset, None, str] = UNSET
    middlename: Union[Unset, None, str] = UNSET
    surname: Union[Unset, None, str] = UNSET
    ni_number: Union[Unset, None, str] = UNSET
    tax_code: Union[Unset, None, str] = UNSET
    date_of_birth: Union[Unset, datetime.date] = UNSET
    join_date: Union[Unset, datetime.date] = UNSET
    leave_date: Union[Unset, None, datetime.date] = UNSET
    is_director: Union[Unset, bool] = UNSET
    ni_and_stat_payments_lines: Union[
        Unset, None, List[P11NiAndStatPaymentsLine]
    ] = UNSET
    totals_line: Union[Unset, P11NiAndStatPaymentsTotalsLine] = UNSET
    ni_summary: Union[Unset, None, List[P11DetailedNiValues]] = UNSET
    paye_lines: Union[Unset, None, List[P11PayeLine]] = UNSET
    paye_totals_line: Union[Unset, P11PayeTotalsLine] = UNSET
    paye_summary: Union[Unset, P11PayeSummary] = UNSET
    report: Union[Unset, Report] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employer_name = self.employer_name
        payroll_code = self.payroll_code
        employer_office_no = self.employer_office_no
        employer_paye_ref = self.employer_paye_ref
        firstname = self.firstname
        middlename = self.middlename
        surname = self.surname
        ni_number = self.ni_number
        tax_code = self.tax_code
        date_of_birth: Union[Unset, str] = UNSET
        if not isinstance(self.date_of_birth, Unset):
            date_of_birth = self.date_of_birth.isoformat()

        join_date: Union[Unset, str] = UNSET
        if not isinstance(self.join_date, Unset):
            join_date = self.join_date.isoformat()

        leave_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.leave_date, Unset):
            leave_date = self.leave_date.isoformat() if self.leave_date else None

        is_director = self.is_director
        ni_and_stat_payments_lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.ni_and_stat_payments_lines, Unset):
            if self.ni_and_stat_payments_lines is None:
                ni_and_stat_payments_lines = None
            else:
                ni_and_stat_payments_lines = []
                for (
                    ni_and_stat_payments_lines_item_data
                ) in self.ni_and_stat_payments_lines:
                    ni_and_stat_payments_lines_item = (
                        ni_and_stat_payments_lines_item_data.to_dict()
                    )

                    ni_and_stat_payments_lines.append(ni_and_stat_payments_lines_item)

        totals_line: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.totals_line, Unset):
            totals_line = self.totals_line.to_dict()

        ni_summary: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.ni_summary, Unset):
            if self.ni_summary is None:
                ni_summary = None
            else:
                ni_summary = []
                for ni_summary_item_data in self.ni_summary:
                    ni_summary_item = ni_summary_item_data.to_dict()

                    ni_summary.append(ni_summary_item)

        paye_lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.paye_lines, Unset):
            if self.paye_lines is None:
                paye_lines = None
            else:
                paye_lines = []
                for paye_lines_item_data in self.paye_lines:
                    paye_lines_item = paye_lines_item_data.to_dict()

                    paye_lines.append(paye_lines_item)

        paye_totals_line: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.paye_totals_line, Unset):
            paye_totals_line = self.paye_totals_line.to_dict()

        paye_summary: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.paye_summary, Unset):
            paye_summary = self.paye_summary.to_dict()

        report: Union[Unset, str] = UNSET
        if not isinstance(self.report, Unset):
            report = self.report.value

        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employer_name is not UNSET:
            field_dict["employerName"] = employer_name
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if employer_office_no is not UNSET:
            field_dict["employerOfficeNo"] = employer_office_no
        if employer_paye_ref is not UNSET:
            field_dict["employerPayeRef"] = employer_paye_ref
        if firstname is not UNSET:
            field_dict["firstname"] = firstname
        if middlename is not UNSET:
            field_dict["middlename"] = middlename
        if surname is not UNSET:
            field_dict["surname"] = surname
        if ni_number is not UNSET:
            field_dict["niNumber"] = ni_number
        if tax_code is not UNSET:
            field_dict["taxCode"] = tax_code
        if date_of_birth is not UNSET:
            field_dict["dateOfBirth"] = date_of_birth
        if join_date is not UNSET:
            field_dict["joinDate"] = join_date
        if leave_date is not UNSET:
            field_dict["leaveDate"] = leave_date
        if is_director is not UNSET:
            field_dict["isDirector"] = is_director
        if ni_and_stat_payments_lines is not UNSET:
            field_dict["niAndStatPaymentsLines"] = ni_and_stat_payments_lines
        if totals_line is not UNSET:
            field_dict["totalsLine"] = totals_line
        if ni_summary is not UNSET:
            field_dict["niSummary"] = ni_summary
        if paye_lines is not UNSET:
            field_dict["payeLines"] = paye_lines
        if paye_totals_line is not UNSET:
            field_dict["payeTotalsLine"] = paye_totals_line
        if paye_summary is not UNSET:
            field_dict["payeSummary"] = paye_summary
        if report is not UNSET:
            field_dict["report"] = report
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employer_name = d.pop("employerName", UNSET)

        payroll_code = d.pop("payrollCode", UNSET)

        employer_office_no = d.pop("employerOfficeNo", UNSET)

        employer_paye_ref = d.pop("employerPayeRef", UNSET)

        firstname = d.pop("firstname", UNSET)

        middlename = d.pop("middlename", UNSET)

        surname = d.pop("surname", UNSET)

        ni_number = d.pop("niNumber", UNSET)

        tax_code = d.pop("taxCode", UNSET)

        _date_of_birth = d.pop("dateOfBirth", UNSET)
        date_of_birth: Union[Unset, datetime.date]
        if isinstance(_date_of_birth, Unset):
            date_of_birth = UNSET
        else:
            date_of_birth = isoparse(_date_of_birth).date()

        _join_date = d.pop("joinDate", UNSET)
        join_date: Union[Unset, datetime.date]
        if isinstance(_join_date, Unset):
            join_date = UNSET
        else:
            join_date = isoparse(_join_date).date()

        _leave_date = d.pop("leaveDate", UNSET)
        leave_date: Union[Unset, None, datetime.date]
        if _leave_date is None:
            leave_date = None
        elif isinstance(_leave_date, Unset):
            leave_date = UNSET
        else:
            leave_date = isoparse(_leave_date).date()

        is_director = d.pop("isDirector", UNSET)

        ni_and_stat_payments_lines = []
        _ni_and_stat_payments_lines = d.pop("niAndStatPaymentsLines", UNSET)
        for ni_and_stat_payments_lines_item_data in _ni_and_stat_payments_lines or []:
            ni_and_stat_payments_lines_item = P11NiAndStatPaymentsLine.from_dict(
                ni_and_stat_payments_lines_item_data
            )

            ni_and_stat_payments_lines.append(ni_and_stat_payments_lines_item)

        _totals_line = d.pop("totalsLine", UNSET)
        totals_line: Union[Unset, P11NiAndStatPaymentsTotalsLine]
        if isinstance(_totals_line, Unset):
            totals_line = UNSET
        else:
            totals_line = P11NiAndStatPaymentsTotalsLine.from_dict(_totals_line)

        ni_summary = []
        _ni_summary = d.pop("niSummary", UNSET)
        for ni_summary_item_data in _ni_summary or []:
            ni_summary_item = P11DetailedNiValues.from_dict(ni_summary_item_data)

            ni_summary.append(ni_summary_item)

        paye_lines = []
        _paye_lines = d.pop("payeLines", UNSET)
        for paye_lines_item_data in _paye_lines or []:
            paye_lines_item = P11PayeLine.from_dict(paye_lines_item_data)

            paye_lines.append(paye_lines_item)

        _paye_totals_line = d.pop("payeTotalsLine", UNSET)
        paye_totals_line: Union[Unset, P11PayeTotalsLine]
        if isinstance(_paye_totals_line, Unset):
            paye_totals_line = UNSET
        else:
            paye_totals_line = P11PayeTotalsLine.from_dict(_paye_totals_line)

        _paye_summary = d.pop("payeSummary", UNSET)
        paye_summary: Union[Unset, P11PayeSummary]
        if isinstance(_paye_summary, Unset):
            paye_summary = UNSET
        else:
            paye_summary = P11PayeSummary.from_dict(_paye_summary)

        _report = d.pop("report", UNSET)
        report: Union[Unset, Report]
        if isinstance(_report, Unset):
            report = UNSET
        else:
            report = Report(_report)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        p11_detailed = cls(
            employer_name=employer_name,
            payroll_code=payroll_code,
            employer_office_no=employer_office_no,
            employer_paye_ref=employer_paye_ref,
            firstname=firstname,
            middlename=middlename,
            surname=surname,
            ni_number=ni_number,
            tax_code=tax_code,
            date_of_birth=date_of_birth,
            join_date=join_date,
            leave_date=leave_date,
            is_director=is_director,
            ni_and_stat_payments_lines=ni_and_stat_payments_lines,
            totals_line=totals_line,
            ni_summary=ni_summary,
            paye_lines=paye_lines,
            paye_totals_line=paye_totals_line,
            paye_summary=paye_summary,
            report=report,
            tax_year=tax_year,
        )

        return p11_detailed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.p11d_car_free_fuel_withdrawn import P11DCarFreeFuelWithdrawn
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11DCar")


@attr.s(auto_attribs=True)
class P11DCar:
    """
    Attributes:
        make (Union[Unset, None, str]):
        registered (Union[Unset, None, str]):
        avail_from (Union[Unset, None, str]):
        avail_to (Union[Unset, None, str]):
        cc (Union[Unset, None, str]):
        fuel (Union[Unset, None, str]):
        c_o2 (Union[Unset, None, str]):
        zero_emission_mileage (Union[Unset, None, str]):
        no_app_co2_fig (Union[Unset, None, str]):
        list_ (Union[Unset, None, str]):
        accs (Union[Unset, None, str]):
        cap_cont (Union[Unset, None, str]):
        priv_use_pmt (Union[Unset, None, str]):
        fuel_withdrawn (Union[Unset, P11DCarFreeFuelWithdrawn]):
        cash_equiv_or_relevant_amt (Union[Unset, None, str]):
        fuel_cash_equiv_or_relevant_amt (Union[Unset, None, str]):
    """

    make: Union[Unset, None, str] = UNSET
    registered: Union[Unset, None, str] = UNSET
    avail_from: Union[Unset, None, str] = UNSET
    avail_to: Union[Unset, None, str] = UNSET
    cc: Union[Unset, None, str] = UNSET
    fuel: Union[Unset, None, str] = UNSET
    c_o2: Union[Unset, None, str] = UNSET
    zero_emission_mileage: Union[Unset, None, str] = UNSET
    no_app_co2_fig: Union[Unset, None, str] = UNSET
    list_: Union[Unset, None, str] = UNSET
    accs: Union[Unset, None, str] = UNSET
    cap_cont: Union[Unset, None, str] = UNSET
    priv_use_pmt: Union[Unset, None, str] = UNSET
    fuel_withdrawn: Union[Unset, P11DCarFreeFuelWithdrawn] = UNSET
    cash_equiv_or_relevant_amt: Union[Unset, None, str] = UNSET
    fuel_cash_equiv_or_relevant_amt: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        make = self.make
        registered = self.registered
        avail_from = self.avail_from
        avail_to = self.avail_to
        cc = self.cc
        fuel = self.fuel
        c_o2 = self.c_o2
        zero_emission_mileage = self.zero_emission_mileage
        no_app_co2_fig = self.no_app_co2_fig
        list_ = self.list_
        accs = self.accs
        cap_cont = self.cap_cont
        priv_use_pmt = self.priv_use_pmt
        fuel_withdrawn: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.fuel_withdrawn, Unset):
            fuel_withdrawn = self.fuel_withdrawn.to_dict()

        cash_equiv_or_relevant_amt = self.cash_equiv_or_relevant_amt
        fuel_cash_equiv_or_relevant_amt = self.fuel_cash_equiv_or_relevant_amt

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if make is not UNSET:
            field_dict["make"] = make
        if registered is not UNSET:
            field_dict["registered"] = registered
        if avail_from is not UNSET:
            field_dict["availFrom"] = avail_from
        if avail_to is not UNSET:
            field_dict["availTo"] = avail_to
        if cc is not UNSET:
            field_dict["cc"] = cc
        if fuel is not UNSET:
            field_dict["fuel"] = fuel
        if c_o2 is not UNSET:
            field_dict["cO2"] = c_o2
        if zero_emission_mileage is not UNSET:
            field_dict["zeroEmissionMileage"] = zero_emission_mileage
        if no_app_co2_fig is not UNSET:
            field_dict["noAppCO2Fig"] = no_app_co2_fig
        if list_ is not UNSET:
            field_dict["list"] = list_
        if accs is not UNSET:
            field_dict["accs"] = accs
        if cap_cont is not UNSET:
            field_dict["capCont"] = cap_cont
        if priv_use_pmt is not UNSET:
            field_dict["privUsePmt"] = priv_use_pmt
        if fuel_withdrawn is not UNSET:
            field_dict["fuelWithdrawn"] = fuel_withdrawn
        if cash_equiv_or_relevant_amt is not UNSET:
            field_dict["cashEquivOrRelevantAmt"] = cash_equiv_or_relevant_amt
        if fuel_cash_equiv_or_relevant_amt is not UNSET:
            field_dict["fuelCashEquivOrRelevantAmt"] = fuel_cash_equiv_or_relevant_amt

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        make = d.pop("make", UNSET)

        registered = d.pop("registered", UNSET)

        avail_from = d.pop("availFrom", UNSET)

        avail_to = d.pop("availTo", UNSET)

        cc = d.pop("cc", UNSET)

        fuel = d.pop("fuel", UNSET)

        c_o2 = d.pop("cO2", UNSET)

        zero_emission_mileage = d.pop("zeroEmissionMileage", UNSET)

        no_app_co2_fig = d.pop("noAppCO2Fig", UNSET)

        list_ = d.pop("list", UNSET)

        accs = d.pop("accs", UNSET)

        cap_cont = d.pop("capCont", UNSET)

        priv_use_pmt = d.pop("privUsePmt", UNSET)

        _fuel_withdrawn = d.pop("fuelWithdrawn", UNSET)
        fuel_withdrawn: Union[Unset, P11DCarFreeFuelWithdrawn]
        if isinstance(_fuel_withdrawn, Unset):
            fuel_withdrawn = UNSET
        else:
            fuel_withdrawn = P11DCarFreeFuelWithdrawn.from_dict(_fuel_withdrawn)

        cash_equiv_or_relevant_amt = d.pop("cashEquivOrRelevantAmt", UNSET)

        fuel_cash_equiv_or_relevant_amt = d.pop("fuelCashEquivOrRelevantAmt", UNSET)

        p11d_car = cls(
            make=make,
            registered=registered,
            avail_from=avail_from,
            avail_to=avail_to,
            cc=cc,
            fuel=fuel,
            c_o2=c_o2,
            zero_emission_mileage=zero_emission_mileage,
            no_app_co2_fig=no_app_co2_fig,
            list_=list_,
            accs=accs,
            cap_cont=cap_cont,
            priv_use_pmt=priv_use_pmt,
            fuel_withdrawn=fuel_withdrawn,
            cash_equiv_or_relevant_amt=cash_equiv_or_relevant_amt,
            fuel_cash_equiv_or_relevant_amt=fuel_cash_equiv_or_relevant_amt,
        )

        return p11d_car



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UtmInfo")


@attr.s(auto_attribs=True)
class UtmInfo:
    """
    Attributes:
        source (Union[Unset, None, str]):
        medium (Union[Unset, None, str]):
        term (Union[Unset, None, str]):
        content (Union[Unset, None, str]):
        campaign (Union[Unset, None, str]):
    """

    source: Union[Unset, None, str] = UNSET
    medium: Union[Unset, None, str] = UNSET
    term: Union[Unset, None, str] = UNSET
    content: Union[Unset, None, str] = UNSET
    campaign: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        source = self.source
        medium = self.medium
        term = self.term
        content = self.content
        campaign = self.campaign

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if source is not UNSET:
            field_dict["source"] = source
        if medium is not UNSET:
            field_dict["medium"] = medium
        if term is not UNSET:
            field_dict["term"] = term
        if content is not UNSET:
            field_dict["content"] = content
        if campaign is not UNSET:
            field_dict["campaign"] = campaign

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        source = d.pop("source", UNSET)

        medium = d.pop("medium", UNSET)

        term = d.pop("term", UNSET)

        content = d.pop("content", UNSET)

        campaign = d.pop("campaign", UNSET)

        utm_info = cls(
            source=source,
            medium=medium,
            term=term,
            content=content,
            campaign=campaign,
        )

        return utm_info



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="EpsFinalSubmission")


@attr.s(auto_attribs=True)
class EpsFinalSubmission:
    """Used on an EPS to declare a Final Submission

    Attributes:
        is_final_submission (Union[Unset, bool]):
        because_scheme_ceased (Union[Unset, bool]):
        date_ceased (Union[Unset, None, datetime.date]):
    """

    is_final_submission: Union[Unset, bool] = UNSET
    because_scheme_ceased: Union[Unset, bool] = UNSET
    date_ceased: Union[Unset, None, datetime.date] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_final_submission = self.is_final_submission
        because_scheme_ceased = self.because_scheme_ceased
        date_ceased: Union[Unset, None, str] = UNSET
        if not isinstance(self.date_ceased, Unset):
            date_ceased = self.date_ceased.isoformat() if self.date_ceased else None

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_final_submission is not UNSET:
            field_dict["isFinalSubmission"] = is_final_submission
        if because_scheme_ceased is not UNSET:
            field_dict["becauseSchemeCeased"] = because_scheme_ceased
        if date_ceased is not UNSET:
            field_dict["dateCeased"] = date_ceased

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_final_submission = d.pop("isFinalSubmission", UNSET)

        because_scheme_ceased = d.pop("becauseSchemeCeased", UNSET)

        _date_ceased = d.pop("dateCeased", UNSET)
        date_ceased: Union[Unset, None, datetime.date]
        if _date_ceased is None:
            date_ceased = None
        elif isinstance(_date_ceased, Unset):
            date_ceased = UNSET
        else:
            date_ceased = isoparse(_date_ceased).date()

        eps_final_submission = cls(
            is_final_submission=is_final_submission,
            because_scheme_ceased=because_scheme_ceased,
            date_ceased=date_ceased,
        )

        return eps_final_submission



from typing import Any, Dict, List, Union

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.pay_options_import import PayOptionsImport
from ...models.pay_periods import PayPeriods
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    json_body: List[PayOptionsImport],
    ordinal: Union[Unset, None, int] = 1,
    lines_only: Union[Unset, None, bool] = False,
    append: Union[Unset, None, bool] = False,
    update_existing: Union[Unset, None, bool] = False,
    validate_only: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/payrun/{payPeriod}/importpay".format(
        client.base_url, employerId=employer_id, payPeriod=pay_period
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["ordinal"] = ordinal

    params["linesOnly"] = lines_only

    params["append"] = append

    params["updateExisting"] = update_existing

    params["validateOnly"] = validate_only

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = []
    for json_body_item_data in json_body:
        json_body_item = json_body_item_data.to_dict()

        json_json_body.append(json_body_item)

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    json_body: List[PayOptionsImport],
    ordinal: Union[Unset, None, int] = 1,
    lines_only: Union[Unset, None, bool] = False,
    append: Union[Unset, None, bool] = False,
    update_existing: Union[Unset, None, bool] = False,
    validate_only: Union[Unset, None, bool] = False,
) -> Response[Any]:
    """Import Pay

     Takes a list PayOptionsImport and updates the currently open payrun to use the amounts given.
    You must have an open payrun for the PayPeriod specified and all payroll codes submitted must match
    an employee on the payrun.
    Any lines imported for a PayRunEntry will replace existing lines for that PayRunEntry.

    Args:
        employer_id (str):
        pay_period (PayPeriods):
        ordinal (Union[Unset, None, int]):  Default: 1.
        lines_only (Union[Unset, None, bool]):
        append (Union[Unset, None, bool]):
        update_existing (Union[Unset, None, bool]):
        validate_only (Union[Unset, None, bool]):
        json_body (List[PayOptionsImport]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        pay_period=pay_period,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
        lines_only=lines_only,
        append=append,
        update_existing=update_existing,
        validate_only=validate_only,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    json_body: List[PayOptionsImport],
    ordinal: Union[Unset, None, int] = 1,
    lines_only: Union[Unset, None, bool] = False,
    append: Union[Unset, None, bool] = False,
    update_existing: Union[Unset, None, bool] = False,
    validate_only: Union[Unset, None, bool] = False,
) -> Response[Any]:
    """Import Pay

     Takes a list PayOptionsImport and updates the currently open payrun to use the amounts given.
    You must have an open payrun for the PayPeriod specified and all payroll codes submitted must match
    an employee on the payrun.
    Any lines imported for a PayRunEntry will replace existing lines for that PayRunEntry.

    Args:
        employer_id (str):
        pay_period (PayPeriods):
        ordinal (Union[Unset, None, int]):  Default: 1.
        lines_only (Union[Unset, None, bool]):
        append (Union[Unset, None, bool]):
        update_existing (Union[Unset, None, bool]):
        validate_only (Union[Unset, None, bool]):
        json_body (List[PayOptionsImport]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        pay_period=pay_period,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
        lines_only=lines_only,
        append=append,
        update_existing=update_existing,
        validate_only=validate_only,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.fps_car import FpsCar
from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsBenefit")


@attr.s(auto_attribs=True)
class FpsBenefit:
    """
    Attributes:
        car (Union[Unset, None, List[FpsCar]]):
    """

    car: Union[Unset, None, List[FpsCar]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        car: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.car, Unset):
            if self.car is None:
                car = None
            else:
                car = []
                for car_item_data in self.car:
                    car_item = car_item_data.to_dict()

                    car.append(car_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if car is not UNSET:
            field_dict["car"] = car

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        car = []
        _car = d.pop("car", UNSET)
        for car_item_data in _car or []:
            car_item = FpsCar.from_dict(car_item_data)

            car.append(car_item)

        fps_benefit = cls(
            car=car,
        )

        return fps_benefit



from enum import Enum


class ExternalDataProviderType(str, Enum):
    PENSIONSCHEMES = "PensionSchemes"
    ACCOUNTING = "Accounting"
    PAYMENTS = "Payments"
    HR = "Hr"
    TIMEANDATTENDANCE = "TimeAndAttendance"
    EMPLOYEEPORTAL = "EmployeePortal"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.address import Address
from ..models.rti_contact import RtiContact
from ..types import UNSET, Unset

T = TypeVar("T", bound="RtiAgent")


@attr.s(auto_attribs=True)
class RtiAgent:
    """
    Attributes:
        agent_id (Union[Unset, None, str]):
        company (Union[Unset, None, str]):
        address (Union[Unset, Address]):
        contact (Union[Unset, RtiContact]):
    """

    agent_id: Union[Unset, None, str] = UNSET
    company: Union[Unset, None, str] = UNSET
    address: Union[Unset, Address] = UNSET
    contact: Union[Unset, RtiContact] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        agent_id = self.agent_id
        company = self.company
        address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.to_dict()

        contact: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.contact, Unset):
            contact = self.contact.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if agent_id is not UNSET:
            field_dict["agentId"] = agent_id
        if company is not UNSET:
            field_dict["company"] = company
        if address is not UNSET:
            field_dict["address"] = address
        if contact is not UNSET:
            field_dict["contact"] = contact

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        agent_id = d.pop("agentId", UNSET)

        company = d.pop("company", UNSET)

        _address = d.pop("address", UNSET)
        address: Union[Unset, Address]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = Address.from_dict(_address)

        _contact = d.pop("contact", UNSET)
        contact: Union[Unset, RtiContact]
        if isinstance(_contact, Unset):
            contact = UNSET
        else:
            contact = RtiContact.from_dict(_contact)

        rti_agent = cls(
            agent_id=agent_id,
            company=company,
            address=address,
            contact=contact,
        )

        return rti_agent



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.background_task_status import BackgroundTaskStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExternalProviderConversation")


@attr.s(auto_attribs=True)
class ExternalProviderConversation:
    """This model is used to provide details of a data exchange with an ExternalDataProvider

    Attributes:
        title (Union[Unset, None, str]):
        content_type (Union[Unset, None, str]):
        url (Union[Unset, None, str]):
        request (Union[Unset, None, str]):
        response (Union[Unset, None, str]):
        status (Union[Unset, BackgroundTaskStatus]):
        status_message (Union[Unset, None, str]):
    """

    title: Union[Unset, None, str] = UNSET
    content_type: Union[Unset, None, str] = UNSET
    url: Union[Unset, None, str] = UNSET
    request: Union[Unset, None, str] = UNSET
    response: Union[Unset, None, str] = UNSET
    status: Union[Unset, BackgroundTaskStatus] = UNSET
    status_message: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        title = self.title
        content_type = self.content_type
        url = self.url
        request = self.request
        response = self.response
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        status_message = self.status_message

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if title is not UNSET:
            field_dict["title"] = title
        if content_type is not UNSET:
            field_dict["contentType"] = content_type
        if url is not UNSET:
            field_dict["url"] = url
        if request is not UNSET:
            field_dict["request"] = request
        if response is not UNSET:
            field_dict["response"] = response
        if status is not UNSET:
            field_dict["status"] = status
        if status_message is not UNSET:
            field_dict["statusMessage"] = status_message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        title = d.pop("title", UNSET)

        content_type = d.pop("contentType", UNSET)

        url = d.pop("url", UNSET)

        request = d.pop("request", UNSET)

        response = d.pop("response", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, BackgroundTaskStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = BackgroundTaskStatus(_status)

        status_message = d.pop("statusMessage", UNSET)

        external_provider_conversation = cls(
            title=title,
            content_type=content_type,
            url=url,
            request=request,
            response=response,
            status=status,
            status_message=status_message,
        )

        return external_provider_conversation



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisEmployeeIdentity")


@attr.s(auto_attribs=True)
class PapdisEmployeeIdentity:
    """
    Attributes:
        employee_id (Union[Unset, None, str]): [readonly]
        birth_date (Union[Unset, datetime.date]): [readonly]
        gender (Union[Unset, None, str]): [readonly]
        national_insurance_number (Union[Unset, None, str]): [readonly]
        employment_start_date (Union[Unset, datetime.date]): [readonly]
    """

    employee_id: Union[Unset, None, str] = UNSET
    birth_date: Union[Unset, datetime.date] = UNSET
    gender: Union[Unset, None, str] = UNSET
    national_insurance_number: Union[Unset, None, str] = UNSET
    employment_start_date: Union[Unset, datetime.date] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee_id = self.employee_id
        birth_date: Union[Unset, str] = UNSET
        if not isinstance(self.birth_date, Unset):
            birth_date = self.birth_date.isoformat()

        gender = self.gender
        national_insurance_number = self.national_insurance_number
        employment_start_date: Union[Unset, str] = UNSET
        if not isinstance(self.employment_start_date, Unset):
            employment_start_date = self.employment_start_date.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee_id is not UNSET:
            field_dict["employeeId"] = employee_id
        if birth_date is not UNSET:
            field_dict["birthDate"] = birth_date
        if gender is not UNSET:
            field_dict["gender"] = gender
        if national_insurance_number is not UNSET:
            field_dict["nationalInsuranceNumber"] = national_insurance_number
        if employment_start_date is not UNSET:
            field_dict["employmentStartDate"] = employment_start_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employee_id = d.pop("employeeId", UNSET)

        _birth_date = d.pop("birthDate", UNSET)
        birth_date: Union[Unset, datetime.date]
        if isinstance(_birth_date, Unset):
            birth_date = UNSET
        else:
            birth_date = isoparse(_birth_date).date()

        gender = d.pop("gender", UNSET)

        national_insurance_number = d.pop("nationalInsuranceNumber", UNSET)

        _employment_start_date = d.pop("employmentStartDate", UNSET)
        employment_start_date: Union[Unset, datetime.date]
        if isinstance(_employment_start_date, Unset):
            employment_start_date = UNSET
        else:
            employment_start_date = isoparse(_employment_start_date).date()

        papdis_employee_identity = cls(
            employee_id=employee_id,
            birth_date=birth_date,
            gender=gender,
            national_insurance_number=national_insurance_number,
            employment_start_date=employment_start_date,
        )

        return papdis_employee_identity



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="DirectorshipDetails")


@attr.s(auto_attribs=True)
class DirectorshipDetails:
    """
    Attributes:
        is_director (Union[Unset, bool]):
        start_date (Union[Unset, None, datetime.date]):
        leave_date (Union[Unset, None, datetime.date]):
        ni_alternative_method (Union[Unset, bool]):
    """

    is_director: Union[Unset, bool] = UNSET
    start_date: Union[Unset, None, datetime.date] = UNSET
    leave_date: Union[Unset, None, datetime.date] = UNSET
    ni_alternative_method: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_director = self.is_director
        start_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat() if self.start_date else None

        leave_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.leave_date, Unset):
            leave_date = self.leave_date.isoformat() if self.leave_date else None

        ni_alternative_method = self.ni_alternative_method

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_director is not UNSET:
            field_dict["isDirector"] = is_director
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if leave_date is not UNSET:
            field_dict["leaveDate"] = leave_date
        if ni_alternative_method is not UNSET:
            field_dict["niAlternativeMethod"] = ni_alternative_method

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_director = d.pop("isDirector", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, None, datetime.date]
        if _start_date is None:
            start_date = None
        elif isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date).date()

        _leave_date = d.pop("leaveDate", UNSET)
        leave_date: Union[Unset, None, datetime.date]
        if _leave_date is None:
            leave_date = None
        elif isinstance(_leave_date, Unset):
            leave_date = UNSET
        else:
            leave_date = isoparse(_leave_date).date()

        ni_alternative_method = d.pop("niAlternativeMethod", UNSET)

        directorship_details = cls(
            is_director=is_director,
            start_date=start_date,
            leave_date=leave_date,
            ni_alternative_method=ni_alternative_method,
        )

        return directorship_details



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.role import Role
from ..types import UNSET, Unset

T = TypeVar("T", bound="EmployerItem")


@attr.s(auto_attribs=True)
class EmployerItem:
    """
    Attributes:
        is_owner (Union[Unset, bool]):
        role (Union[Unset, Role]):
        id (Union[Unset, str]):
        name (Union[Unset, None, str]):
        metadata (Union[Unset, Any]):
        url (Union[Unset, None, str]):
    """

    is_owner: Union[Unset, bool] = UNSET
    role: Union[Unset, Role] = UNSET
    id: Union[Unset, str] = UNSET
    name: Union[Unset, None, str] = UNSET
    metadata: Union[Unset, Any] = UNSET
    url: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_owner = self.is_owner
        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        id = self.id
        name = self.name
        metadata = self.metadata
        url = self.url

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_owner is not UNSET:
            field_dict["isOwner"] = is_owner
        if role is not UNSET:
            field_dict["role"] = role
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if url is not UNSET:
            field_dict["url"] = url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_owner = d.pop("isOwner", UNSET)

        _role = d.pop("role", UNSET)
        role: Union[Unset, Role]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = Role(_role)

        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        metadata = d.pop("metadata", UNSET)

        url = d.pop("url", UNSET)

        employer_item = cls(
            is_owner=is_owner,
            role=role,
            id=id,
            name=name,
            metadata=metadata,
            url=url,
        )

        return employer_item



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisEmployeeAssessment")


@attr.s(auto_attribs=True)
class PapdisEmployeeAssessment:
    """
    Attributes:
        assessment_code (Union[Unset, int]): [readonly]
        event_code (Union[Unset, int]): [readonly]
        event_date (Union[Unset, None, datetime.date]): [readonly]
        statutory_letter_code (Union[Unset, None, str]): [readonly]
        is_individual_already_member_of_qps (Union[Unset, bool]): [readonly]
        deferral_date (Union[Unset, None, datetime.date]): [readonly]
        enrolment_communications_issued_date (Union[Unset, None, datetime.date]): [readonly]
        worker_exclusion_code (Union[Unset, None, str]): [readonly]
        reenrolment_indicator (Union[Unset, bool]): [readonly]
        opt_out_window_end_date (Union[Unset, None, datetime.date]): [readonly]
    """

    assessment_code: Union[Unset, int] = UNSET
    event_code: Union[Unset, int] = UNSET
    event_date: Union[Unset, None, datetime.date] = UNSET
    statutory_letter_code: Union[Unset, None, str] = UNSET
    is_individual_already_member_of_qps: Union[Unset, bool] = UNSET
    deferral_date: Union[Unset, None, datetime.date] = UNSET
    enrolment_communications_issued_date: Union[Unset, None, datetime.date] = UNSET
    worker_exclusion_code: Union[Unset, None, str] = UNSET
    reenrolment_indicator: Union[Unset, bool] = UNSET
    opt_out_window_end_date: Union[Unset, None, datetime.date] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        assessment_code = self.assessment_code
        event_code = self.event_code
        event_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.event_date, Unset):
            event_date = self.event_date.isoformat() if self.event_date else None

        statutory_letter_code = self.statutory_letter_code
        is_individual_already_member_of_qps = self.is_individual_already_member_of_qps
        deferral_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.deferral_date, Unset):
            deferral_date = (
                self.deferral_date.isoformat() if self.deferral_date else None
            )

        enrolment_communications_issued_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.enrolment_communications_issued_date, Unset):
            enrolment_communications_issued_date = (
                self.enrolment_communications_issued_date.isoformat()
                if self.enrolment_communications_issued_date
                else None
            )

        worker_exclusion_code = self.worker_exclusion_code
        reenrolment_indicator = self.reenrolment_indicator
        opt_out_window_end_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.opt_out_window_end_date, Unset):
            opt_out_window_end_date = (
                self.opt_out_window_end_date.isoformat()
                if self.opt_out_window_end_date
                else None
            )

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if assessment_code is not UNSET:
            field_dict["assessmentCode"] = assessment_code
        if event_code is not UNSET:
            field_dict["eventCode"] = event_code
        if event_date is not UNSET:
            field_dict["eventDate"] = event_date
        if statutory_letter_code is not UNSET:
            field_dict["statutoryLetterCode"] = statutory_letter_code
        if is_individual_already_member_of_qps is not UNSET:
            field_dict[
                "isIndividualAlreadyMemberOfQPS"
            ] = is_individual_already_member_of_qps
        if deferral_date is not UNSET:
            field_dict["deferralDate"] = deferral_date
        if enrolment_communications_issued_date is not UNSET:
            field_dict[
                "enrolmentCommunicationsIssuedDate"
            ] = enrolment_communications_issued_date
        if worker_exclusion_code is not UNSET:
            field_dict["workerExclusionCode"] = worker_exclusion_code
        if reenrolment_indicator is not UNSET:
            field_dict["reenrolmentIndicator"] = reenrolment_indicator
        if opt_out_window_end_date is not UNSET:
            field_dict["optOutWindowEndDate"] = opt_out_window_end_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        assessment_code = d.pop("assessmentCode", UNSET)

        event_code = d.pop("eventCode", UNSET)

        _event_date = d.pop("eventDate", UNSET)
        event_date: Union[Unset, None, datetime.date]
        if _event_date is None:
            event_date = None
        elif isinstance(_event_date, Unset):
            event_date = UNSET
        else:
            event_date = isoparse(_event_date).date()

        statutory_letter_code = d.pop("statutoryLetterCode", UNSET)

        is_individual_already_member_of_qps = d.pop(
            "isIndividualAlreadyMemberOfQPS", UNSET
        )

        _deferral_date = d.pop("deferralDate", UNSET)
        deferral_date: Union[Unset, None, datetime.date]
        if _deferral_date is None:
            deferral_date = None
        elif isinstance(_deferral_date, Unset):
            deferral_date = UNSET
        else:
            deferral_date = isoparse(_deferral_date).date()

        _enrolment_communications_issued_date = d.pop(
            "enrolmentCommunicationsIssuedDate", UNSET
        )
        enrolment_communications_issued_date: Union[Unset, None, datetime.date]
        if _enrolment_communications_issued_date is None:
            enrolment_communications_issued_date = None
        elif isinstance(_enrolment_communications_issued_date, Unset):
            enrolment_communications_issued_date = UNSET
        else:
            enrolment_communications_issued_date = isoparse(
                _enrolment_communications_issued_date
            ).date()

        worker_exclusion_code = d.pop("workerExclusionCode", UNSET)

        reenrolment_indicator = d.pop("reenrolmentIndicator", UNSET)

        _opt_out_window_end_date = d.pop("optOutWindowEndDate", UNSET)
        opt_out_window_end_date: Union[Unset, None, datetime.date]
        if _opt_out_window_end_date is None:
            opt_out_window_end_date = None
        elif isinstance(_opt_out_window_end_date, Unset):
            opt_out_window_end_date = UNSET
        else:
            opt_out_window_end_date = isoparse(_opt_out_window_end_date).date()

        papdis_employee_assessment = cls(
            assessment_code=assessment_code,
            event_code=event_code,
            event_date=event_date,
            statutory_letter_code=statutory_letter_code,
            is_individual_already_member_of_qps=is_individual_already_member_of_qps,
            deferral_date=deferral_date,
            enrolment_communications_issued_date=enrolment_communications_issued_date,
            worker_exclusion_code=worker_exclusion_code,
            reenrolment_indicator=reenrolment_indicator,
            opt_out_window_end_date=opt_out_window_end_date,
        )

        return papdis_employee_assessment



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="RtiEmployeeName")


@attr.s(auto_attribs=True)
class RtiEmployeeName:
    """
    Attributes:
        ttl (Union[Unset, None, str]):
        fore (Union[Unset, None, List[str]]):
        initials (Union[Unset, None, str]):
        sur (Union[Unset, None, str]):
    """

    ttl: Union[Unset, None, str] = UNSET
    fore: Union[Unset, None, List[str]] = UNSET
    initials: Union[Unset, None, str] = UNSET
    sur: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        ttl = self.ttl
        fore: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.fore, Unset):
            if self.fore is None:
                fore = None
            else:
                fore = self.fore

        initials = self.initials
        sur = self.sur

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if ttl is not UNSET:
            field_dict["ttl"] = ttl
        if fore is not UNSET:
            field_dict["fore"] = fore
        if initials is not UNSET:
            field_dict["initials"] = initials
        if sur is not UNSET:
            field_dict["sur"] = sur

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ttl = d.pop("ttl", UNSET)

        fore = cast(List[str], d.pop("fore", UNSET))

        initials = d.pop("initials", UNSET)

        sur = d.pop("sur", UNSET)

        rti_employee_name = cls(
            ttl=ttl,
            fore=fore,
            initials=initials,
            sur=sur,
        )

        return rti_employee_name



from typing import Any, Dict

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...types import Response


def _get_kwargs(
    employer_id: str,
    employee_id: str,
    id: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/employees/{employeeId}/autoenrolment/assessments/{id}/letter/sent".format(
        client.base_url, employerId=employer_id, employeeId=employee_id, id=id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    employer_id: str,
    employee_id: str,
    id: str,
    *,
    client: Client,
) -> Response[Any]:
    """Mark Letter as Sent

     Update an Assessment to mark the letter as sent.
    This is done automatically if you send the letter via email so you'd
    only need to use this is you are printing/posting the letter.

    Args:
        employer_id (str):
        employee_id (str):
        id (str):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        employee_id=employee_id,
        id=id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    employer_id: str,
    employee_id: str,
    id: str,
    *,
    client: Client,
) -> Response[Any]:
    """Mark Letter as Sent

     Update an Assessment to mark the letter as sent.
    This is done automatically if you send the letter via email so you'd
    only need to use this is you are printing/posting the letter.

    Args:
        employer_id (str):
        employee_id (str):
        id (str):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        employee_id=employee_id,
        id=id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProcessingNote")


@attr.s(auto_attribs=True)
class ProcessingNote:
    """
    Attributes:
        note (str):
        pay_run_entry_id (Union[Unset, None, str]): The Id of the payrunEntry this ProcessingNote relates to, if any
        user (Union[Unset, Item]):
        pay_run (Union[Unset, Item]):
        date (Union[Unset, datetime.date]):
        document_count (Union[Unset, int]): [readonly] The number of attachments associated with this model
        documents (Union[Unset, None, List[Item]]): [readonly] The attachments associated with this model
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    note: str
    pay_run_entry_id: Union[Unset, None, str] = UNSET
    user: Union[Unset, Item] = UNSET
    pay_run: Union[Unset, Item] = UNSET
    date: Union[Unset, datetime.date] = UNSET
    document_count: Union[Unset, int] = UNSET
    documents: Union[Unset, None, List[Item]] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        note = self.note
        pay_run_entry_id = self.pay_run_entry_id
        user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user, Unset):
            user = self.user.to_dict()

        pay_run: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pay_run, Unset):
            pay_run = self.pay_run.to_dict()

        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        document_count = self.document_count
        documents: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.documents, Unset):
            if self.documents is None:
                documents = None
            else:
                documents = []
                for documents_item_data in self.documents:
                    documents_item = documents_item_data.to_dict()

                    documents.append(documents_item)

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "note": note,
            }
        )
        if pay_run_entry_id is not UNSET:
            field_dict["payRunEntryId"] = pay_run_entry_id
        if user is not UNSET:
            field_dict["user"] = user
        if pay_run is not UNSET:
            field_dict["payRun"] = pay_run
        if date is not UNSET:
            field_dict["date"] = date
        if document_count is not UNSET:
            field_dict["documentCount"] = document_count
        if documents is not UNSET:
            field_dict["documents"] = documents
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        note = d.pop("note")

        pay_run_entry_id = d.pop("payRunEntryId", UNSET)

        _user = d.pop("user", UNSET)
        user: Union[Unset, Item]
        if isinstance(_user, Unset):
            user = UNSET
        else:
            user = Item.from_dict(_user)

        _pay_run = d.pop("payRun", UNSET)
        pay_run: Union[Unset, Item]
        if isinstance(_pay_run, Unset):
            pay_run = UNSET
        else:
            pay_run = Item.from_dict(_pay_run)

        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.date]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        document_count = d.pop("documentCount", UNSET)

        documents = []
        _documents = d.pop("documents", UNSET)
        for documents_item_data in _documents or []:
            documents_item = Item.from_dict(documents_item_data)

            documents.append(documents_item)

        id = d.pop("id", UNSET)

        processing_note = cls(
            note=note,
            pay_run_entry_id=pay_run_entry_id,
            user=user,
            pay_run=pay_run,
            date=date,
            document_count=document_count,
            documents=documents,
            id=id,
        )

        return processing_note



from typing import Any, Dict, List, Union

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    *,
    client: Client,
    json_body: List[str],
    on_hold: Union[Unset, None, bool] = UNSET,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/employees/onhold".format(
        client.base_url, employerId=employer_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["onHold"] = on_hold

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    employer_id: str,
    *,
    client: Client,
    json_body: List[str],
    on_hold: Union[Unset, None, bool] = UNSET,
) -> Response[Any]:
    """Mark as On Hold

     Used to mark multiple employees as On Hold or not On Hold.
    The body should contain an array of Ids to identify the employees to update.

    Args:
        employer_id (str):
        on_hold (Union[Unset, None, bool]):
        json_body (List[str]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        client=client,
        json_body=json_body,
        on_hold=on_hold,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    employer_id: str,
    *,
    client: Client,
    json_body: List[str],
    on_hold: Union[Unset, None, bool] = UNSET,
) -> Response[Any]:
    """Mark as On Hold

     Used to mark multiple employees as On Hold or not On Hold.
    The body should contain an array of Ids to identify the employees to update.

    Args:
        employer_id (str):
        on_hold (Union[Unset, None, bool]):
        json_body (List[str]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        client=client,
        json_body=json_body,
        on_hold=on_hold,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from typing import Any, Dict, Optional, cast

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...types import Response


def _get_kwargs(
    employer_id: str,
    *,
    client: Client,
    json_body: str,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/dps/notices/xml".format(
        client.base_url, employerId=employer_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[int]:
    if response.status_code == 200:
        response_200 = cast(int, response.json())
        return response_200
    return raise_staffology_exception(response)


def _build_response(*, response: httpx.Response) -> Response[int]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    *,
    client: Client,
    json_body: str,
) -> Response[int]:
    """Parse notices from XML

     Checks the supplied XML string for notices. They're then processed in the same was as if received
    from HMRC.
    Only used for testing the parsing of DPS XML.
    Returns an integer showing how many new notices were found

    Args:
        employer_id (str):
        json_body (str):

    Returns:
        Response[int]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    *,
    client: Client,
    json_body: str,
) -> Optional[int]:
    """Parse notices from XML

     Checks the supplied XML string for notices. They're then processed in the same was as if received
    from HMRC.
    Only used for testing the parsing of DPS XML.
    Returns an integer showing how many new notices were found

    Args:
        employer_id (str):
        json_body (str):

    Returns:
        Response[int]
    """

    return sync_detailed(
        employer_id=employer_id,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    *,
    client: Client,
    json_body: str,
) -> Response[int]:
    """Parse notices from XML

     Checks the supplied XML string for notices. They're then processed in the same was as if received
    from HMRC.
    Only used for testing the parsing of DPS XML.
    Returns an integer showing how many new notices were found

    Args:
        employer_id (str):
        json_body (str):

    Returns:
        Response[int]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    *,
    client: Client,
    json_body: str,
) -> Optional[int]:
    """Parse notices from XML

     Checks the supplied XML string for notices. They're then processed in the same was as if received
    from HMRC.
    Only used for testing the parsing of DPS XML.
    Returns an integer showing how many new notices were found

    Args:
        employer_id (str):
        json_body (str):

    Returns:
        Response[int]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.pay_code import PayCode
from ..types import UNSET, Unset

T = TypeVar("T", bound="FullSummaryOfPayReportLine")


@attr.s(auto_attribs=True)
class FullSummaryOfPayReportLine:
    """
    Attributes:
        pay_code (Union[Unset, PayCode]): Each PayLine has a Code. The Code will match the Code property of a PayCode.
            The PayCode that is used determines how the amount is treated with regards to tax, NI and pensions
        value (Union[Unset, float]):
        er_value (Union[Unset, None, float]):
        qty (Union[Unset, None, float]):
        head_count (Union[Unset, int]):
    """

    pay_code: Union[Unset, PayCode] = UNSET
    value: Union[Unset, float] = UNSET
    er_value: Union[Unset, None, float] = UNSET
    qty: Union[Unset, None, float] = UNSET
    head_count: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pay_code: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pay_code, Unset):
            pay_code = self.pay_code.to_dict()

        value = self.value
        er_value = self.er_value
        qty = self.qty
        head_count = self.head_count

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if pay_code is not UNSET:
            field_dict["payCode"] = pay_code
        if value is not UNSET:
            field_dict["value"] = value
        if er_value is not UNSET:
            field_dict["erValue"] = er_value
        if qty is not UNSET:
            field_dict["qty"] = qty
        if head_count is not UNSET:
            field_dict["headCount"] = head_count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _pay_code = d.pop("payCode", UNSET)
        pay_code: Union[Unset, PayCode]
        if isinstance(_pay_code, Unset):
            pay_code = UNSET
        else:
            pay_code = PayCode.from_dict(_pay_code)

        value = d.pop("value", UNSET)

        er_value = d.pop("erValue", UNSET)

        qty = d.pop("qty", UNSET)

        head_count = d.pop("headCount", UNSET)

        full_summary_of_pay_report_line = cls(
            pay_code=pay_code,
            value=value,
            er_value=er_value,
            qty=qty,
            head_count=head_count,
        )

        return full_summary_of_pay_report_line



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.overseas_secondment_status import OverseasSecondmentStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="OverseasEmployerDetails")


@attr.s(auto_attribs=True)
class OverseasEmployerDetails:
    """
    Attributes:
        overseas_employer (Union[Unset, bool]):
        overseas_secondment_status (Union[Unset, OverseasSecondmentStatus]):
        eea_citizen (Union[Unset, bool]):
        epm_6_scheme (Union[Unset, bool]):
    """

    overseas_employer: Union[Unset, bool] = UNSET
    overseas_secondment_status: Union[Unset, OverseasSecondmentStatus] = UNSET
    eea_citizen: Union[Unset, bool] = UNSET
    epm_6_scheme: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        overseas_employer = self.overseas_employer
        overseas_secondment_status: Union[Unset, str] = UNSET
        if not isinstance(self.overseas_secondment_status, Unset):
            overseas_secondment_status = self.overseas_secondment_status.value

        eea_citizen = self.eea_citizen
        epm_6_scheme = self.epm_6_scheme

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if overseas_employer is not UNSET:
            field_dict["overseasEmployer"] = overseas_employer
        if overseas_secondment_status is not UNSET:
            field_dict["overseasSecondmentStatus"] = overseas_secondment_status
        if eea_citizen is not UNSET:
            field_dict["eeaCitizen"] = eea_citizen
        if epm_6_scheme is not UNSET:
            field_dict["epm6Scheme"] = epm_6_scheme

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        overseas_employer = d.pop("overseasEmployer", UNSET)

        _overseas_secondment_status = d.pop("overseasSecondmentStatus", UNSET)
        overseas_secondment_status: Union[Unset, OverseasSecondmentStatus]
        if isinstance(_overseas_secondment_status, Unset):
            overseas_secondment_status = UNSET
        else:
            overseas_secondment_status = OverseasSecondmentStatus(
                _overseas_secondment_status
            )

        eea_citizen = d.pop("eeaCitizen", UNSET)

        epm_6_scheme = d.pop("epm6Scheme", UNSET)

        overseas_employer_details = cls(
            overseas_employer=overseas_employer,
            overseas_secondment_status=overseas_secondment_status,
            eea_citizen=eea_citizen,
            epm_6_scheme=epm_6_scheme,
        )

        return overseas_employer_details



from enum import Enum


class Role(str, Enum):
    ADMIN = "Admin"
    EDITOR = "Editor"
    REVIEWER = "Reviewer"
    PAYROLLCLIENT = "PayrollClient"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class AeEmployeeState(str, Enum):
    AUTOMATIC = "Automatic"
    OPTOUT = "OptOut"
    OPTIN = "OptIn"
    VOLUNTARYJOINER = "VoluntaryJoiner"
    CONTRACTUALPENSION = "ContractualPension"
    CEASEDMEMBERSHIP = "CeasedMembership"
    LEAVER = "Leaver"
    EXCLUDED = "Excluded"
    ENROL = "Enrol"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.cis_return import CisReturn
from ..models.emp_refs import EmpRefs
from ..models.gov_talk_submission import GovTalkSubmission
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="Cis300")


@attr.s(auto_attribs=True)
class Cis300:
    """
    Attributes:
        tax_month (Union[Unset, int]):
        employee_count (Union[Unset, int]):
        employment_status_declaration (Union[Unset, bool]):
        verification_declaration (Union[Unset, bool]):
        information_correct_declaration (Union[Unset, bool]):
        inactivity_declaration (Union[Unset, bool]):
        cis_return (Union[Unset, CisReturn]):
        i_rmark (Union[Unset, None, str]):
        xml (Union[Unset, None, str]):
        tax_year (Union[Unset, TaxYear]):
        employer_references (Union[Unset, EmpRefs]):
        gov_talk_submission (Union[Unset, GovTalkSubmission]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    tax_month: Union[Unset, int] = UNSET
    employee_count: Union[Unset, int] = UNSET
    employment_status_declaration: Union[Unset, bool] = UNSET
    verification_declaration: Union[Unset, bool] = UNSET
    information_correct_declaration: Union[Unset, bool] = UNSET
    inactivity_declaration: Union[Unset, bool] = UNSET
    cis_return: Union[Unset, CisReturn] = UNSET
    i_rmark: Union[Unset, None, str] = UNSET
    xml: Union[Unset, None, str] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET
    employer_references: Union[Unset, EmpRefs] = UNSET
    gov_talk_submission: Union[Unset, GovTalkSubmission] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        tax_month = self.tax_month
        employee_count = self.employee_count
        employment_status_declaration = self.employment_status_declaration
        verification_declaration = self.verification_declaration
        information_correct_declaration = self.information_correct_declaration
        inactivity_declaration = self.inactivity_declaration
        cis_return: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.cis_return, Unset):
            cis_return = self.cis_return.to_dict()

        i_rmark = self.i_rmark
        xml = self.xml
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        employer_references: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer_references, Unset):
            employer_references = self.employer_references.to_dict()

        gov_talk_submission: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.gov_talk_submission, Unset):
            gov_talk_submission = self.gov_talk_submission.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if tax_month is not UNSET:
            field_dict["taxMonth"] = tax_month
        if employee_count is not UNSET:
            field_dict["employeeCount"] = employee_count
        if employment_status_declaration is not UNSET:
            field_dict["employmentStatusDeclaration"] = employment_status_declaration
        if verification_declaration is not UNSET:
            field_dict["verificationDeclaration"] = verification_declaration
        if information_correct_declaration is not UNSET:
            field_dict[
                "informationCorrectDeclaration"
            ] = information_correct_declaration
        if inactivity_declaration is not UNSET:
            field_dict["inactivityDeclaration"] = inactivity_declaration
        if cis_return is not UNSET:
            field_dict["cisReturn"] = cis_return
        if i_rmark is not UNSET:
            field_dict["iRmark"] = i_rmark
        if xml is not UNSET:
            field_dict["xml"] = xml
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if employer_references is not UNSET:
            field_dict["employerReferences"] = employer_references
        if gov_talk_submission is not UNSET:
            field_dict["govTalkSubmission"] = gov_talk_submission
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        tax_month = d.pop("taxMonth", UNSET)

        employee_count = d.pop("employeeCount", UNSET)

        employment_status_declaration = d.pop("employmentStatusDeclaration", UNSET)

        verification_declaration = d.pop("verificationDeclaration", UNSET)

        information_correct_declaration = d.pop("informationCorrectDeclaration", UNSET)

        inactivity_declaration = d.pop("inactivityDeclaration", UNSET)

        _cis_return = d.pop("cisReturn", UNSET)
        cis_return: Union[Unset, CisReturn]
        if isinstance(_cis_return, Unset):
            cis_return = UNSET
        else:
            cis_return = CisReturn.from_dict(_cis_return)

        i_rmark = d.pop("iRmark", UNSET)

        xml = d.pop("xml", UNSET)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        _employer_references = d.pop("employerReferences", UNSET)
        employer_references: Union[Unset, EmpRefs]
        if isinstance(_employer_references, Unset):
            employer_references = UNSET
        else:
            employer_references = EmpRefs.from_dict(_employer_references)

        _gov_talk_submission = d.pop("govTalkSubmission", UNSET)
        gov_talk_submission: Union[Unset, GovTalkSubmission]
        if isinstance(_gov_talk_submission, Unset):
            gov_talk_submission = UNSET
        else:
            gov_talk_submission = GovTalkSubmission.from_dict(_gov_talk_submission)

        id = d.pop("id", UNSET)

        cis_300 = cls(
            tax_month=tax_month,
            employee_count=employee_count,
            employment_status_declaration=employment_status_declaration,
            verification_declaration=verification_declaration,
            information_correct_declaration=information_correct_declaration,
            inactivity_declaration=inactivity_declaration,
            cis_return=cis_return,
            i_rmark=i_rmark,
            xml=xml,
            tax_year=tax_year,
            employer_references=employer_references,
            gov_talk_submission=gov_talk_submission,
            id=id,
        )

        return cis_300



from typing import Any, Dict, List, Optional

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.item import Item
from ...models.tax_year import TaxYear
from ...types import Response


def _get_kwargs(
    employer_id: str,
    id: str,
    tax_year: TaxYear,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = (
        "{}/employers/{employerId}/pensionschemes/{id}/contributions/{taxYear}".format(
            client.base_url, employerId=employer_id, id=id, taxYear=tax_year
        )
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[Item]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Item.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    return raise_staffology_exception(response)


def _build_response(*, response: httpx.Response) -> Response[List[Item]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    id: str,
    tax_year: TaxYear,
    *,
    client: Client,
) -> Response[List[Item]]:
    """List Contributions

     Returns a list of Items that tell you which PayRuns contain contributions to this scheme and the Url
    to the relevant report.
    IF the scheme is connected to an ExternalDataProvider then the status of the submission will also be
    included.

    Args:
        employer_id (str):
        id (str):
        tax_year (TaxYear):

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        id=id,
        tax_year=tax_year,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    id: str,
    tax_year: TaxYear,
    *,
    client: Client,
) -> Optional[List[Item]]:
    """List Contributions

     Returns a list of Items that tell you which PayRuns contain contributions to this scheme and the Url
    to the relevant report.
    IF the scheme is connected to an ExternalDataProvider then the status of the submission will also be
    included.

    Args:
        employer_id (str):
        id (str):
        tax_year (TaxYear):

    Returns:
        Response[List[Item]]
    """

    return sync_detailed(
        employer_id=employer_id,
        id=id,
        tax_year=tax_year,
        client=client,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    id: str,
    tax_year: TaxYear,
    *,
    client: Client,
) -> Response[List[Item]]:
    """List Contributions

     Returns a list of Items that tell you which PayRuns contain contributions to this scheme and the Url
    to the relevant report.
    IF the scheme is connected to an ExternalDataProvider then the status of the submission will also be
    included.

    Args:
        employer_id (str):
        id (str):
        tax_year (TaxYear):

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        id=id,
        tax_year=tax_year,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    id: str,
    tax_year: TaxYear,
    *,
    client: Client,
) -> Optional[List[Item]]:
    """List Contributions

     Returns a list of Items that tell you which PayRuns contain contributions to this scheme and the Url
    to the relevant report.
    IF the scheme is connected to an ExternalDataProvider then the status of the submission will also be
    included.

    Args:
        employer_id (str):
        id (str):
        tax_year (TaxYear):

    Returns:
        Response[List[Item]]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            id=id,
            tax_year=tax_year,
            client=client,
        )
    ).parsed
